#
import pytest
from load_one_data import dump_diff, dump_diff_text, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_arabic_category_label

data_empty_value = {
    "Category:Scheduled multi-sport events": "تصنيف:أحداث رياضية متعددة مقررة",
    "Category:Defunct multi-sport events": "تصنيف:أحداث رياضية متعددة سابقة",
    "Category:Disabled multi-sport events": "تصنيف:أحداث رياضية متعددة معاقون",
    "Category:Timor-Leste at multi-sport events": "تصنيف:أحداث رياضية متعددة في تيمور الشرقية",
    "Category:Wikipedia categories named after multi-sport events": "تصنيف:تصنيفات سميت بأسماء أحداث رياضية متعددة",
    "Category:Youth multi-sport events": "تصنيف:أحداث رياضية متعددة شبابية",
}

data_0 = {
    "Category:1896 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1896",
    "Category:1900 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1900",
    "Category:1904 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1904",
    "Category:1906 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1906",
    "Category:1908 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1908",
    "Category:1912 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1912",
    "Category:1916 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1916",
    "Category:1920 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1920",
    "Category:1924 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1924",
    "Category:1926 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1926",
    "Category:1928 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1928",
    "Category:1930 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1930",
    "Category:1932 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1932",
    "Category:1933 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1933",
    "Category:1934 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1934",
    "Category:1935 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1935",
    "Category:1936 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1936",
    "Category:1938 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1938",
    "Category:1940 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1940",
    "Category:1944 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1944",
    "Category:1946 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1946",
    "Category:1948 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1948",
    "Category:1950 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1950",
    "Category:1951 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1951",
    "Category:1952 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1952",
    "Category:1953 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1953",
    "Category:1954 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1954",
    "Category:1955 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1955",
    "Category:1956 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1956",
    "Category:1957 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1957",
    "Category:1958 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1958",
    "Category:1959 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1959",
    "Category:1960 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1960",
    "Category:1961 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1961",
    "Category:1962 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1962",
    "Category:1963 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1963",
    "Category:1964 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1964",
    "Category:1965 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1965",
    "Category:1966 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1966",
    "Category:1967 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1967",
    "Category:1968 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1968",
    "Category:1969 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1969",
    "Category:1970 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1970",
    "Category:1971 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1971",
    "Category:1972 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1972",
    "Category:1973 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1973",
    "Category:1974 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1974",
    "Category:1975 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1975",
    "Category:1976 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1976",
    "Category:1977 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1977",
    "Category:1978 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1978",
    "Category:1979 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1979",
    "Category:1980 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1980",
    "Category:1981 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1981",
    "Category:1982 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1982",
    "Category:1983 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1983",
    "Category:1984 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1984",
    "Category:1985 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1985",
    "Category:1986 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1986",
    "Category:1987 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1987",
    "Category:1988 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1988",
    "Category:1989 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1989",
    "Category:1990 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1990",
    "Category:1991 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1991",
    "Category:1992 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1992",
    "Category:1993 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1993",
    "Category:1994 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1994",
    "Category:1995 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1995",
    "Category:1996 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1996",
    "Category:1997 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1997",
    "Category:1998 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1998",
    "Category:1999 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 1999",
    "Category:2000 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2000",
    "Category:2001 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2001",
    "Category:2002 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2002",
    "Category:2003 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2003",
    "Category:2004 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2004",
    "Category:2005 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2005",
    "Category:2006 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2006",
    "Category:2007 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2007",
    "Category:2008 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2008",
    "Category:2009 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2009",
    "Category:2010 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2010",
    "Category:2011 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2011",
    "Category:2012 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2012",
    "Category:2013 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2013",
    "Category:2014 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2014",
    "Category:2015 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2015",
    "Category:2016 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2016",
    "Category:2017 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2017",
    "Category:2018 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2018",
    "Category:2019 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2019",
    "Category:2020 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2020",
    "Category:2021 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2021",
    "Category:2022 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2022",
    "Category:2024 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2024",
    "Category:2025 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2025",
    "Category:2026 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2026",
    "Category:2028 in multi-sport events": "تصنيف:الأحداث الرياضية المتعددة في 2028",
}


data_1 = {
    "Category:Albania at winter multi-sport events": "تصنيف:ألبانيا في أحداث رياضية متعددة شتوية",
    "Category:Algeria at winter multi-sport events": "تصنيف:الجزائر في أحداث رياضية متعددة شتوية",
    "Category:American Samoa at winter multi-sport events": "تصنيف:ساموا الأمريكية في أحداث رياضية متعددة شتوية",
    "Category:Argentina at winter multi-sport events": "تصنيف:الأرجنتين في أحداث رياضية متعددة شتوية",
    "Category:Australia at winter multi-sport events": "تصنيف:أستراليا في أحداث رياضية متعددة شتوية",
    "Category:Austria at winter multi-sport events": "تصنيف:النمسا في أحداث رياضية متعددة شتوية",
    "Category:Brazil at winter multi-sport events": "تصنيف:البرازيل في أحداث رياضية متعددة شتوية",
    "Category:Canada at winter multi-sport events": "تصنيف:كندا في أحداث رياضية متعددة شتوية",
    "Category:China at winter multi-sport events": "تصنيف:الصين في أحداث رياضية متعددة شتوية",
    "Category:Czech Republic at winter multi-sport events": "تصنيف:التشيك في أحداث رياضية متعددة شتوية",
    "Category:East Germany at winter multi-sport events": "تصنيف:ألمانيا الشرقية في أحداث رياضية متعددة شتوية",
    "Category:France at winter multi-sport events": "تصنيف:فرنسا في أحداث رياضية متعددة شتوية",
    "Category:Germany at winter multi-sport events": "تصنيف:ألمانيا في أحداث رياضية متعددة شتوية",
    "Category:Hong Kong at winter multi-sport events": "تصنيف:هونغ كونغ في أحداث رياضية متعددة شتوية",
    "Category:India at winter multi-sport events": "تصنيف:الهند في أحداث رياضية متعددة شتوية",
    "Category:Iran at winter multi-sport events": "تصنيف:إيران في أحداث رياضية متعددة شتوية",
    "Category:Israel at winter multi-sport events": "تصنيف:إسرائيل في أحداث رياضية متعددة شتوية",
    "Category:Italy at winter multi-sport events": "تصنيف:إيطاليا في أحداث رياضية متعددة شتوية",
    "Category:Japan at winter multi-sport events": "تصنيف:اليابان في أحداث رياضية متعددة شتوية",
    "Category:Kazakhstan at winter multi-sport events": "تصنيف:كازاخستان في أحداث رياضية متعددة شتوية",
    "Category:Kyrgyzstan at winter multi-sport events": "تصنيف:قيرغيزستان في أحداث رياضية متعددة شتوية",
    "Category:Lebanon at winter multi-sport events": "تصنيف:لبنان في أحداث رياضية متعددة شتوية",
    "Category:Lithuania at winter multi-sport events": "تصنيف:ليتوانيا في أحداث رياضية متعددة شتوية",
    "Category:Malaysia at winter multi-sport events": "تصنيف:ماليزيا في أحداث رياضية متعددة شتوية",
    "Category:Mongolia at winter multi-sport events": "تصنيف:منغوليا في أحداث رياضية متعددة شتوية",
    "Category:Multi-sport events": "تصنيف:أحداث رياضية متعددة",
    "Category:Multi-sport events-related lists": "تصنيف:قوائم متعلقة بأحداث رياضية متعددة",
    "Category:Multi-sport events bids": "تصنيف:ترشيحات أحداث رياضية متعددة",
    "Category:Multi-sport events by country": "تصنيف:أحداث رياضية متعددة حسب البلد",
    "Category:Multi-sport events by year": "تصنيف:أحداث رياضية متعددة حسب السنة",
    "Category:Multi-sport events cancelled due to the COVID-19 pandemic": "تصنيف:أحداث رياضية متعددة ألغيت بسبب جائحة فيروس كورونا",
    "Category:Multi-sport events in Africa by country": "تصنيف:أحداث رياضية متعددة في إفريقيا حسب البلد",
    "Category:Multi-sport events in Andorra": "تصنيف:أحداث رياضية متعددة في أندورا",
    "Category:Multi-sport events in Argentina": "تصنيف:أحداث رياضية متعددة في الأرجنتين",
    "Category:Multi-sport events in Asia": "تصنيف:أحداث رياضية متعددة في آسيا",
    "Category:Multi-sport events in Asia by country": "تصنيف:أحداث رياضية متعددة في آسيا حسب البلد",
    "Category:Multi-sport events in Australia": "تصنيف:أحداث رياضية متعددة في أستراليا",
    "Category:Multi-sport events in Austria": "تصنيف:أحداث رياضية متعددة في النمسا",
    "Category:Multi-sport events in Azerbaijan": "تصنيف:أحداث رياضية متعددة في أذربيجان",
    "Category:Multi-sport events in Bahrain": "تصنيف:أحداث رياضية متعددة في البحرين",
    "Category:Multi-sport events in Belgium": "تصنيف:أحداث رياضية متعددة في بلجيكا",
    "Category:Multi-sport events in Bolivia": "تصنيف:أحداث رياضية متعددة في بوليفيا",
    "Category:Multi-sport events in Brazil": "تصنيف:أحداث رياضية متعددة في البرازيل",
    "Category:Multi-sport events in Bulgaria": "تصنيف:أحداث رياضية متعددة في بلغاريا",
    "Category:Multi-sport events in Canada": "تصنيف:أحداث رياضية متعددة في كندا",
    "Category:Multi-sport events in Chile": "تصنيف:أحداث رياضية متعددة في تشيلي",
    "Category:Multi-sport events in China": "تصنيف:أحداث رياضية متعددة في الصين",
    "Category:Multi-sport events in Colombia": "تصنيف:أحداث رياضية متعددة في كولومبيا",
    "Category:Multi-sport events in Croatia": "تصنيف:أحداث رياضية متعددة في كرواتيا",
    "Category:Multi-sport events in Cuba": "تصنيف:أحداث رياضية متعددة في كوبا",
    "Category:Multi-sport events in Cyprus": "تصنيف:أحداث رياضية متعددة في قبرص",
    "Category:Multi-sport events in Czechoslovakia": "تصنيف:أحداث رياضية متعددة في تشيكوسلوفاكيا",
    "Category:Multi-sport events in Denmark": "تصنيف:أحداث رياضية متعددة في الدنمارك",
    "Category:Multi-sport events in East Germany": "تصنيف:أحداث رياضية متعددة في ألمانيا الشرقية",
    "Category:Multi-sport events in Ecuador": "تصنيف:أحداث رياضية متعددة في الإكوادور",
    "Category:Multi-sport events in El Salvador": "تصنيف:أحداث رياضية متعددة في السلفادور",
    "Category:Multi-sport events in Europe": "تصنيف:أحداث رياضية متعددة في أوروبا",
    "Category:Multi-sport events in Europe by country": "تصنيف:أحداث رياضية متعددة في أوروبا حسب البلد",
    "Category:Multi-sport events in Finland": "تصنيف:أحداث رياضية متعددة في فنلندا",
    "Category:Multi-sport events in France": "تصنيف:أحداث رياضية متعددة في فرنسا",
    "Category:Multi-sport events in Germany": "تصنيف:أحداث رياضية متعددة في ألمانيا",
    "Category:Multi-sport events in Greece": "تصنيف:أحداث رياضية متعددة في اليونان",
    "Category:Multi-sport events in Hungary": "تصنيف:أحداث رياضية متعددة في المجر",
    "Category:Multi-sport events in Iceland": "تصنيف:أحداث رياضية متعددة في آيسلندا",
    "Category:Multi-sport events in India": "تصنيف:أحداث رياضية متعددة في الهند",
    "Category:Multi-sport events in Indonesia": "تصنيف:أحداث رياضية متعددة في إندونيسيا",
    "Category:Multi-sport events in insular areas of the United States": "تصنيف:أحداث رياضية متعددة في المناطق المعزولة في الولايات المتحدة",
    "Category:Multi-sport events in Iran": "تصنيف:أحداث رياضية متعددة في إيران",
    "Category:Multi-sport events in Israel": "تصنيف:أحداث رياضية متعددة في إسرائيل",
    "Category:Multi-sport events in Italy": "تصنيف:أحداث رياضية متعددة في إيطاليا",
    "Category:Multi-sport events in Jamaica": "تصنيف:أحداث رياضية متعددة في جامايكا",
    "Category:Multi-sport events in Japan": "تصنيف:أحداث رياضية متعددة في اليابان",
    "Category:Multi-sport events in Jordan": "تصنيف:أحداث رياضية متعددة في الأردن",
    "Category:Multi-sport events in Kazakhstan": "تصنيف:أحداث رياضية متعددة في كازاخستان",
    "Category:Multi-sport events in Kenya": "تصنيف:أحداث رياضية متعددة في كينيا",
    "Category:Multi-sport events in Kuwait": "تصنيف:أحداث رياضية متعددة في الكويت",
    "Category:Multi-sport events in Lebanon": "تصنيف:أحداث رياضية متعددة في لبنان",
    "Category:Multi-sport events in Liechtenstein": "تصنيف:أحداث رياضية متعددة في ليختنشتاين",
    "Category:Multi-sport events in Luxembourg": "تصنيف:أحداث رياضية متعددة في لوكسمبورغ",
    "Category:Multi-sport events in Macau": "تصنيف:أحداث رياضية متعددة في ماكاو",
    "Category:Multi-sport events in Malaysia": "تصنيف:أحداث رياضية متعددة في ماليزيا",
    "Category:Multi-sport events in Malta": "تصنيف:أحداث رياضية متعددة في مالطا",
    "Category:Multi-sport events in Mexico": "تصنيف:أحداث رياضية متعددة في المكسيك",
    "Category:Multi-sport events in Monaco": "تصنيف:أحداث رياضية متعددة في موناكو",
    "Category:Multi-sport events in Mongolia": "تصنيف:أحداث رياضية متعددة في منغوليا",
    "Category:Multi-sport events in Montenegro": "تصنيف:أحداث رياضية متعددة في الجبل الأسود",
    "Category:Multi-sport events in Mozambique": "تصنيف:أحداث رياضية متعددة في موزمبيق",
    "Category:Multi-sport events in New Zealand": "تصنيف:أحداث رياضية متعددة في نيوزيلندا",
    "Category:Multi-sport events in Nigeria": "تصنيف:أحداث رياضية متعددة في نيجيريا",
    "Category:Multi-sport events in North America": "تصنيف:أحداث رياضية متعددة في أمريكا الشمالية",
    "Category:Multi-sport events in North Korea": "تصنيف:أحداث رياضية متعددة في كوريا الشمالية",
    "Category:Multi-sport events in Norway": "تصنيف:أحداث رياضية متعددة في النرويج",
    "Category:Multi-sport events in Oceania": "تصنيف:أحداث رياضية متعددة في أوقيانوسيا",
    "Category:Multi-sport events in Oman": "تصنيف:أحداث رياضية متعددة في سلطنة عمان",
    "Category:Multi-sport events in Panama": "تصنيف:أحداث رياضية متعددة في بنما",
    "Category:Multi-sport events in Peru": "تصنيف:أحداث رياضية متعددة في بيرو",
    "Category:Multi-sport events in Poland": "تصنيف:أحداث رياضية متعددة في بولندا",
    "Category:Multi-sport events in Puerto Rico": "تصنيف:أحداث رياضية متعددة في بورتوريكو",
    "Category:Multi-sport events in Qatar": "تصنيف:أحداث رياضية متعددة في قطر",
    "Category:Multi-sport events in Romania": "تصنيف:أحداث رياضية متعددة في رومانيا",
    "Category:Multi-sport events in Russia": "تصنيف:أحداث رياضية متعددة في روسيا",
    "Category:Multi-sport events in Samoa": "تصنيف:أحداث رياضية متعددة في ساموا",
    "Category:Multi-sport events in San Marino": "تصنيف:أحداث رياضية متعددة في سان مارينو",
    "Category:Multi-sport events in Serbia": "تصنيف:أحداث رياضية متعددة في صربيا",
    "Category:Multi-sport events in Singapore": "تصنيف:أحداث رياضية متعددة في سنغافورة",
    "Category:Multi-sport events in Slovakia": "تصنيف:أحداث رياضية متعددة في سلوفاكيا",
    "Category:Multi-sport events in South America": "تصنيف:أحداث رياضية متعددة في أمريكا الجنوبية",
    "Category:Multi-sport events in South Korea": "تصنيف:أحداث رياضية متعددة في كوريا الجنوبية",
    "Category:Multi-sport events in Spain": "تصنيف:أحداث رياضية متعددة في إسبانيا",
    "Category:Multi-sport events in Sweden": "تصنيف:أحداث رياضية متعددة في السويد",
    "Category:Multi-sport events in Switzerland": "تصنيف:أحداث رياضية متعددة في سويسرا",
    "Category:Multi-sport events in Syria": "تصنيف:أحداث رياضية متعددة في سوريا",
    "Category:Multi-sport events in Taiwan": "تصنيف:أحداث رياضية متعددة في تايوان",
    "Category:Multi-sport events in Thailand": "تصنيف:أحداث رياضية متعددة في تايلاند",
    "Category:Multi-sport events in the Netherlands": "تصنيف:أحداث رياضية متعددة في هولندا",
    "Category:Multi-sport events in the Republic of the Congo": "تصنيف:أحداث رياضية متعددة في جمهورية الكونغو",
    "Category:Multi-sport events in the Soviet Union": "تصنيف:أحداث رياضية متعددة في الاتحاد السوفيتي",
    "Category:Multi-sport events in the United Kingdom": "تصنيف:أحداث رياضية متعددة في المملكة المتحدة",
    "Category:Multi-sport events in the United States": "تصنيف:أحداث رياضية متعددة في الولايات المتحدة",
    "Category:Multi-sport events in Turkey": "تصنيف:أحداث رياضية متعددة في تركيا",
    "Category:Multi-sport events in Turkmenistan": "تصنيف:أحداث رياضية متعددة في تركمانستان",
    "Category:Multi-sport events in Venezuela": "تصنيف:أحداث رياضية متعددة في فنزويلا",
    "Category:Multi-sport events in Vietnam": "تصنيف:أحداث رياضية متعددة في فيتنام",
    "Category:Multi-sport events in West Germany": "تصنيف:أحداث رياضية متعددة في ألمانيا الغربية",
    "Category:Multi-sport events in Yugoslavia": "تصنيف:أحداث رياضية متعددة في يوغسلافيا",
    "Category:Multi-sport events in Zimbabwe": "تصنيف:أحداث رياضية متعددة في زيمبابوي",
    "Category:Multi-sport events templates": "تصنيف:قوالب أحداث رياضية متعددة",
    "Category:Multi-sport events venues": "تصنيف:ملاعب أحداث رياضية متعددة",
    "Category:National multi-sport events": "تصنيف:أحداث رياضية متعددة وطنية",
    "Category:Nations at winter multi-sport events": "تصنيف:بلدان في أحداث رياضية متعددة شتوية",
    "Category:Nepal at winter multi-sport events": "تصنيف:نيبال في أحداث رياضية متعددة شتوية",
    "Category:New Zealand at winter multi-sport events": "تصنيف:نيوزيلندا في أحداث رياضية متعددة شتوية",
    "Category:North Korea at winter multi-sport events": "تصنيف:كوريا الشمالية في أحداث رياضية متعددة شتوية",
    "Category:Pakistan at winter multi-sport events": "تصنيف:باكستان في أحداث رياضية متعددة شتوية",
    "Category:Philippines at winter multi-sport events": "تصنيف:الفلبين في أحداث رياضية متعددة شتوية",
    "Category:Singapore at winter multi-sport events": "تصنيف:سنغافورة في أحداث رياضية متعددة شتوية",
    "Category:South Korea at winter multi-sport events": "تصنيف:كوريا الجنوبية في أحداث رياضية متعددة شتوية",
    "Category:Spain at winter multi-sport events": "تصنيف:إسبانيا في أحداث رياضية متعددة شتوية",
    "Category:Summer multi-sport events": "تصنيف:أحداث رياضية متعددة صيفية",
    "Category:Tajikistan at winter multi-sport events": "تصنيف:طاجيكستان في أحداث رياضية متعددة شتوية",
    "Category:Thailand at winter multi-sport events": "تصنيف:تايلاند في أحداث رياضية متعددة شتوية",
    "Category:Turkey at winter multi-sport events": "تصنيف:تركيا في أحداث رياضية متعددة شتوية",
    "Category:Ukraine at winter multi-sport events": "تصنيف:أوكرانيا في أحداث رياضية متعددة شتوية",
    "Category:United States at winter multi-sport events": "تصنيف:الولايات المتحدة في أحداث رياضية متعددة شتوية",
    "Category:Uzbekistan at winter multi-sport events": "تصنيف:أوزبكستان في أحداث رياضية متعددة شتوية",
    "Category:West Germany at winter multi-sport events": "تصنيف:ألمانيا الغربية في أحداث رياضية متعددة شتوية",
    "Category:Winter multi-sport events by country": "تصنيف:أحداث رياضية متعددة شتوية حسب البلد",
    "Category:Winter multi-sport events in Germany": "تصنيف:أحداث رياضية متعددة شتوية في ألمانيا",
    "Category:Winter multi-sport events in Norway": "تصنيف:أحداث رياضية متعددة شتوية في النرويج",
    "Category:Winter multi-sport events in Russia": "تصنيف:أحداث رياضية متعددة شتوية في روسيا",
    "Category:Winter multi-sport events in the United States": "تصنيف:أحداث رياضية متعددة شتوية في الولايات المتحدة",
    "Category:Multi-sport clubs in Egypt": "تصنيف:أندية متعددة الرياضات في مصر",
    "Category:Multi-sport clubs in Finland": "تصنيف:أندية متعددة الرياضات في فنلندا",
    "Category:Multi-sport clubs in France": "تصنيف:أندية متعددة الرياضات في فرنسا",
    "Category:Multi-sport clubs in Germany": "تصنيف:أندية متعددة الرياضات في ألمانيا",
    "Category:Multi-sport clubs in Greece": "تصنيف:أندية متعددة الرياضات في اليونان",
    "Category:Multi-sport clubs in Iceland": "تصنيف:أندية متعددة الرياضات في آيسلندا",
    "Category:Multi-sport clubs in India": "تصنيف:أندية متعددة الرياضات في الهند",
    "Category:Soviet Union at multi-sport events": "تصنيف:الاتحاد السوفيتي في الأحداث الرياضية المتعددة",
    "Category:Swaziland at multi-sport events": "تصنيف:سوازيلاند في الأحداث الرياضية المتعددة",
    "Category:Wheelchair tennis at multi-sport events": "تصنيف:كرة المضرب على الكراسي المتحركة في الأحداث الرياضية المتعددة",
    "category:soviet union in multi-sport events": "تصنيف:الاتحاد السوفيتي في الأحداث الرياضية المتعددة",
    "Category:Afghanistan at multi-sport events": "تصنيف:أفغانستان في الأحداث الرياضية المتعددة",
    "Category:Algeria at multi-sport events": "تصنيف:الجزائر في الأحداث الرياضية المتعددة",
    "Category:Angola at multi-sport events": "تصنيف:أنغولا في الأحداث الرياضية المتعددة",
    "Category:Antigua and Barbuda at multi-sport events": "تصنيف:أنتيغوا وباربودا في الأحداث الرياضية المتعددة",
    "Category:Bahamas at multi-sport events": "تصنيف:باهاماس في الأحداث الرياضية المتعددة",
    "Category:Bahrain at multi-sport events": "تصنيف:البحرين في الأحداث الرياضية المتعددة",
    "Category:Barbados at multi-sport events": "تصنيف:بربادوس في الأحداث الرياضية المتعددة",
    "Category:Belarus at multi-sport events": "تصنيف:روسيا البيضاء في الأحداث الرياضية المتعددة",
    "Category:Belize at multi-sport events": "تصنيف:بليز في الأحداث الرياضية المتعددة",
    "Category:Benin at multi-sport events": "تصنيف:بنين في الأحداث الرياضية المتعددة",
    "Category:Bermuda at multi-sport events": "تصنيف:برمودا في الأحداث الرياضية المتعددة",
    "Category:Bhutan at multi-sport events": "تصنيف:بوتان في الأحداث الرياضية المتعددة",
    "Category:Burkina Faso at multi-sport events": "تصنيف:بوركينا فاسو في الأحداث الرياضية المتعددة",
    "Category:Burundi at multi-sport events": "تصنيف:بوروندي في الأحداث الرياضية المتعددة",
    "Category:Cambodia at multi-sport events": "تصنيف:كمبوديا في الأحداث الرياضية المتعددة",
    "Category:Cameroon at multi-sport events": "تصنيف:الكاميرون في الأحداث الرياضية المتعددة",
    "Category:Cape Verde at multi-sport events": "تصنيف:الرأس الأخضر في الأحداث الرياضية المتعددة",
    "Category:Central African Republic at multi-sport events": "تصنيف:جمهورية إفريقيا الوسطى في الأحداث الرياضية المتعددة",
    "Category:Chad at multi-sport events": "تصنيف:تشاد في الأحداث الرياضية المتعددة",
    "Category:Chess at multi-sport events": "تصنيف:الشطرنج في الأحداث الرياضية المتعددة",
    "Category:Comoros at multi-sport events": "تصنيف:جزر القمر في الأحداث الرياضية المتعددة",
    "Category:Competitors at multi-sport events": "تصنيف:منافسون في الأحداث الرياضية المتعددة",
    "Category:Competitors at multi-sport events by country": "تصنيف:منافسون في الأحداث الرياضية المتعددة حسب البلد",
    "Category:Competitors at multi-sport events by sport": "تصنيف:منافسون في الأحداث الرياضية المتعددة حسب الرياضة",
    "Category:Competitors at multi-sport events by year": "تصنيف:منافسون في الأحداث الرياضية المتعددة حسب السنة",
    "Category:Czech Republic at multi-sport events": "تصنيف:التشيك في الأحداث الرياضية المتعددة",
    "Category:Dependent territories of the United Kingdom at multi-sport events": "تصنيف:أقاليم ما وراء البحار البريطانية في الأحداث الرياضية المتعددة",
    "Category:Djibouti at multi-sport events": "تصنيف:جيبوتي في الأحداث الرياضية المتعددة",
    "Category:Dominica at multi-sport events": "تصنيف:دومينيكا في الأحداث الرياضية المتعددة",
    "Category:Dominican Republic at multi-sport events": "تصنيف:جمهورية الدومينيكان في الأحداث الرياضية المتعددة",
    "Category:Egypt at multi-sport events": "تصنيف:مصر في الأحداث الرياضية المتعددة",
    "Category:Equatorial Guinea at multi-sport events": "تصنيف:غينيا الاستوائية في الأحداث الرياضية المتعددة",
    "Category:Equestrian sports at multi-sport events": "تصنيف:رياضات فروسية في الأحداث الرياضية المتعددة",
    "Category:Eritrea at multi-sport events": "تصنيف:إرتريا في الأحداث الرياضية المتعددة",
    "Category:Eswatini at multi-sport events": "تصنيف:إسواتيني في الأحداث الرياضية المتعددة",
    "Category:Ethiopia at multi-sport events": "تصنيف:إثيوبيا في الأحداث الرياضية المتعددة",
    "Category:Fiji at multi-sport events": "تصنيف:فيجي في الأحداث الرياضية المتعددة",
    "Category:Gabon at multi-sport events": "تصنيف:الغابون في الأحداث الرياضية المتعددة",
    "Category:Ghana at multi-sport events": "تصنيف:غانا في الأحداث الرياضية المتعددة",
    "Category:Gibraltar at multi-sport events": "تصنيف:جبل طارق في الأحداث الرياضية المتعددة",
    "Category:Go at multi-sport events": "تصنيف:غو في الأحداث الرياضية المتعددة",
    "Category:Grenada at multi-sport events": "تصنيف:غرينادا في الأحداث الرياضية المتعددة",
    "Category:Guinea-Bissau at multi-sport events": "تصنيف:غينيا بيساو في الأحداث الرياضية المتعددة",
    "Category:Guinea at multi-sport events": "تصنيف:غينيا في الأحداث الرياضية المتعددة",
    "Category:Guyana at multi-sport events": "تصنيف:غيانا في الأحداث الرياضية المتعددة",
    "Category:Haiti at multi-sport events": "تصنيف:هايتي في الأحداث الرياضية المتعددة",
    "Category:Insular areas of the United States at multi-sport events": "تصنيف:المناطق المعزولة في الولايات المتحدة في الأحداث الرياضية المتعددة",
    "Category:Iraq at multi-sport events": "تصنيف:العراق في الأحداث الرياضية المتعددة",
    "Category:Ivory Coast at multi-sport events": "تصنيف:ساحل العاج في الأحداث الرياضية المتعددة",
    "Category:Ju-jitsu at multi-sport events": "تصنيف:جوجوتسو في الأحداث الرياضية المتعددة",
    "Category:Judo at multi-sport events": "تصنيف:الجودو في الأحداث الرياضية المتعددة",
    "Category:Kiribati at multi-sport events": "تصنيف:كيريباتي في الأحداث الرياضية المتعددة",
    "Category:Kosovo at multi-sport events": "تصنيف:كوسوفو في الأحداث الرياضية المتعددة",
    "Category:Lesotho at multi-sport events": "تصنيف:ليسوتو في الأحداث الرياضية المتعددة",
    "Category:Liberia at multi-sport events": "تصنيف:ليبيريا في الأحداث الرياضية المتعددة",
    "Category:Libya at multi-sport events": "تصنيف:ليبيا في الأحداث الرياضية المتعددة",
    "Category:Malawi at multi-sport events": "تصنيف:ملاوي في الأحداث الرياضية المتعددة",
    "Category:Maldives at multi-sport events": "تصنيف:جزر المالديف في الأحداث الرياضية المتعددة",
    "Category:Mali at multi-sport events": "تصنيف:مالي في الأحداث الرياضية المتعددة",
    "Category:Marshall Islands at multi-sport events": "تصنيف:جزر مارشال في الأحداث الرياضية المتعددة",
    "Category:Mauritania at multi-sport events": "تصنيف:موريتانيا في الأحداث الرياضية المتعددة",
    "Category:Mauritius at multi-sport events": "تصنيف:موريشيوس في الأحداث الرياضية المتعددة",
    "Category:Medalists at multi-sport events by year": "تصنيف:فائزون بميداليات في الأحداث الرياضية المتعددة حسب السنة",
    "Category:Modern pentathlon at multi-sport events": "تصنيف:الخماسي الحديث في الأحداث الرياضية المتعددة",
    "Category:Moldova at multi-sport events": "تصنيف:مولدوفا في الأحداث الرياضية المتعددة",
    "Category:Morocco at multi-sport events": "تصنيف:المغرب في الأحداث الرياضية المتعددة",
    "Category:Multi-sport events by continent": "تصنيف:أحداث رياضية متعددة حسب القارة",
    "Category:Multi-sport events in Africa": "تصنيف:أحداث رياضية متعددة في إفريقيا",
    "Category:Multi-sport events in Algeria": "تصنيف:أحداث رياضية متعددة في الجزائر",
    "Category:Multi-sport events in Egypt": "تصنيف:أحداث رياضية متعددة في مصر",
    "Category:Multi-sport events in Morocco": "تصنيف:أحداث رياضية متعددة في المغرب",
    "Category:Multi-sport events in Saudi Arabia": "تصنيف:أحداث رياضية متعددة في السعودية",
    "Category:Multi-sport events in Tunisia": "تصنيف:أحداث رياضية متعددة في تونس",
    "Category:Multi-sport events navigational boxes": "تصنيف:صناديق تصفح أحداث رياضية متعددة",
    "Category:Namibia at multi-sport events": "تصنيف:ناميبيا في الأحداث الرياضية المتعددة",
    "Category:Nations at multi-sport events": "تصنيف:بلدان في الأحداث الرياضية المتعددة",
    "Category:Nations at multi-sport events by country": "تصنيف:بلدان في الأحداث الرياضية المتعددة حسب البلد",
    "Category:Netherlands Antilles at multi-sport events": "تصنيف:جزر الأنتيل الهولندية في الأحداث الرياضية المتعددة",
    "Category:North Macedonia at multi-sport events": "تصنيف:مقدونيا الشمالية في الأحداث الرياضية المتعددة",
    "Category:Palestine at multi-sport events": "تصنيف:فلسطين في الأحداث الرياضية المتعددة",
    "Category:Papua New Guinea at multi-sport events": "تصنيف:بابوا غينيا الجديدة في الأحداث الرياضية المتعددة",
    "Category:Paraguay at multi-sport events": "تصنيف:باراغواي في الأحداث الرياضية المتعددة",
    "Category:Philippines at multi-sport events": "تصنيف:الفلبين في الأحداث الرياضية المتعددة",
    "Category:Rwanda at multi-sport events": "تصنيف:رواندا في الأحداث الرياضية المتعددة",
    "Category:Saint Kitts and Nevis at multi-sport events": "تصنيف:سانت كيتس ونيفيس في الأحداث الرياضية المتعددة",
    "Category:Saint Lucia at multi-sport events": "تصنيف:سانت لوسيا في الأحداث الرياضية المتعددة",
    "Category:Saint Vincent and the Grenadines at multi-sport events": "تصنيف:سانت فنسنت وجزر غرينادين في الأحداث الرياضية المتعددة",
    "Category:Senegal at multi-sport events": "تصنيف:السنغال في الأحداث الرياضية المتعددة",
    "Category:Seychelles at multi-sport events": "تصنيف:سيشل في الأحداث الرياضية المتعددة",
    "Category:Skeleton at multi-sport events": "تصنيف:الزلاجات الصدرية في الأحداث الرياضية المتعددة",
    "Category:Slovenia at multi-sport events": "تصنيف:سلوفينيا في الأحداث الرياضية المتعددة",
    "Category:Snooker at multi-sport events": "تصنيف:السنوكر في الأحداث الرياضية المتعددة",
    "Category:Solomon Islands at multi-sport events": "تصنيف:جزر سليمان في الأحداث الرياضية المتعددة",
    "Category:Somalia at multi-sport events": "تصنيف:الصومال في الأحداث الرياضية المتعددة",
    "Category:South Sudan at multi-sport events": "تصنيف:جنوب السودان في الأحداث الرياضية المتعددة",
    "Category:Sport climbing at multi-sport events": "تصنيف:التسلق في الأحداث الرياضية المتعددة",
    "Category:Sudan at multi-sport events": "تصنيف:السودان في الأحداث الرياضية المتعددة",
    "Category:Surfing at multi-sport events": "تصنيف:الركمجة في الأحداث الرياضية المتعددة",
    "Category:Suriname at multi-sport events": "تصنيف:سورينام في الأحداث الرياضية المتعددة",
    "Category:São Tomé and Príncipe at multi-sport events": "تصنيف:ساو تومي وبرينسيب في الأحداث الرياضية المتعددة",
    "Category:Tanzania at multi-sport events": "تصنيف:تنزانيا في الأحداث الرياضية المتعددة",
    "Category:Tennis at multi-sport events": "تصنيف:كرة المضرب في الأحداث الرياضية المتعددة",
    "Category:The Gambia at multi-sport events": "تصنيف:غامبيا في الأحداث الرياضية المتعددة",
    "Category:Togo at multi-sport events": "تصنيف:توغو في الأحداث الرياضية المتعددة",
    "Category:Tonga at multi-sport events": "تصنيف:تونغا في الأحداث الرياضية المتعددة",
    "Category:Trinidad and Tobago at multi-sport events": "تصنيف:ترينيداد وتوباغو في الأحداث الرياضية المتعددة",
    "Category:Tunisia at multi-sport events": "تصنيف:تونس في الأحداث الرياضية المتعددة",
    "Category:Tuvalu at multi-sport events": "تصنيف:توفالو في الأحداث الرياضية المتعددة",
    "Category:Uganda at multi-sport events": "تصنيف:أوغندا في الأحداث الرياضية المتعددة",
    "Category:Ukraine at multi-sport events": "تصنيف:أوكرانيا في الأحداث الرياضية المتعددة",
    "Category:United Arab Emirates at multi-sport events": "تصنيف:الإمارات العربية المتحدة في الأحداث الرياضية المتعددة",
    "Category:Vanuatu at multi-sport events": "تصنيف:فانواتو في الأحداث الرياضية المتعددة",
    "Category:Wheelchair basketball at multi-sport events": "تصنيف:كرة السلة على الكراسي المتحركة في الأحداث الرياضية المتعددة",
    "Category:Winter multi-sport events": "تصنيف:أحداث رياضية متعددة شتوية",
    "Category:Winter multi-sport events in Austria": "تصنيف:أحداث رياضية متعددة شتوية في النمسا",
    "Category:Winter multi-sport events in Canada": "تصنيف:أحداث رياضية متعددة شتوية في كندا",
    "Category:Xiangqi at multi-sport events": "تصنيف:الشطرنج الصيني في الأحداث الرياضية المتعددة",
}

to_test = [
    ("test_multi_sport_0", data_0),
    ("test_multi_sport_1", data_1),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
def test_multi_sport_0(category: str, expected: str) -> None:
    assert resolve_arabic_category_label(category) == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_it(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)
    # dump_same_and_not_same(data, diff_result, name)

    dump_diff(diff_result, name)
    # dump_diff_text(expected, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
This module provides functionality to translate category titles
"""
import functools

from ...helps import len_print, logger
from ...translations import (
    FEMALE_JOBS_BASE_EXTENDED,
    RELIGIOUS_KEYS_PP,
    All_Nat,
    all_country_with_nat,
    all_country_with_nat_ar,
    countries_en_as_nationality_keys,
    jobs_womens_data,
)
from ...translations_formats import MultiDataFormatterBaseV2, format_multi_data_v2
from ..nats_as_country_names import nats_keys_as_country_names
from .utils import fix_keys, nat_and_gender_keys, one_Keys_more_2

countries_en_keys = [x.get("en") for x in all_country_with_nat.values() if x.get("en")]

keys_not_jobs = [
    "women",
    "men",
]


genders_keys_new_under_test: dict[str, str] = {
    # "kidnapped": "مختطفون",
    "fictional": "خياليات",
    # "disabled": "معاقون",
    # "contemporary": "معاصرون",
    # "latin": "لاتينيون",
    # "child": "أطفال",
    # "political": "سياسيون",
    # "religious": "دينيون",
    # "military": "عسكريون",
}
"""
"female {en_job} fictional": "{ar_job} خياليات",
"female {en_job} {en_nat} fictional": "{ar_job} {females} خياليات",
"female {en_nat} fictional": "{females} خياليات",
"female {en_nat} {en_job} fictional": "{ar_job} {females} خياليات",
"fictional {en_nat} female": "{females} خياليات",
"{en_job} female {en_nat} fictional": "{ar_job} {females} خياليات",
"{en_nat} female {en_job} fictional": "{ar_job} {females} خياليات",
"""
genders_keys: dict[str, str] = {
    "blind": "مكفوفات",
    "abolitionists": "مناهضات للعبودية",
    "deaf": "صم",
    "executed": "أعدمن",
    "executed abroad": "أعدمن في الخارج",
    "deafblind": "صم ومكفوفات",
    "killed-in-action": "قتلن في عمليات قتالية",
    "killed in action": "قتلن في عمليات قتالية",
    "murdered abroad": "قتلن في الخارج",
}

genders_keys.update(genders_keys_new_under_test)


def is_false_key(key: str, value: str) -> bool:
    if ("mens" in key.lower() or "men's" in key.lower()) and "رجالية" in value:
        return True

    if key in genders_keys:  # NOTE: under test
        return True

    if RELIGIOUS_KEYS_PP.get(key) or key in keys_not_jobs:
        return True

    not_in_keys = [
        "expatriate",
        "immigrants",
    ]

    # if any(word in key for word in not_in_keys) and not
    if any(word in key for word in not_in_keys):
        return True

    return False


@functools.lru_cache(maxsize=1)
def _load_formatted_data() -> tuple[dict[str, str], dict[str, str]]:
    formatted_data_jobs_with_nat = {
        "{en_nat} female actresses": "ممثلات {females}",
        "{en_nat} actresses": "ممثلات {females}",
        "{en_nat} expatriate female {en_job}": "{ar_job} {females} مغتربات",
        "{en_nat}-american female people": "أمريكيات {females}",
        "{en_nat} female eugenicists": "عالمات {females} متخصصات في تحسين النسل",
        "{en_nat} female politicians who committed suicide": "سياسيات {females} أقدمن على الانتحار",
        "{en_nat} female contemporary artists": "فنانات {females} معاصرات",
        # base keys
        "female {en_nat} people": "{females}",
        "{en_nat} female people": "{females}",
        "female {en_nat}": "{females}",
        "{en_nat} female": "{females}",
    }

    formatted_data_jobs_with_nat.update(nat_and_gender_keys("{en_nat}", "expatriate", "female", "{females} مغتربات"))
    formatted_data_jobs_with_nat.update(nat_and_gender_keys("{en_nat}", "emigrants", "female", "{females} مهاجرات"))

    formatted_data_jobs = {
        # jobs
        # NOTE: "{en_job}": "{ar_job}", Should be used in males bot: [yemeni singers] : "تصنيف:مغنون يمنيون"
        # NOTE: "{en_job}": "{ar_job}", Should be used here to handle womens jobs like: [yemeni actresses] : "تصنيف:ممثلات يمنيات"
        # base keys
        "{en_job}": "{ar_job}",
        "female {en_job}": "{ar_job}",
        "female {en_job} people": "{ar_job}",
        # "{en_job} people": "أعلام {ar_job}",
        "{en_job} people": "{ar_job}",
        # expatriate keys
        "female expatriate {en_job}": "{ar_job} مغتربات",
        "expatriate female {en_job}": "{ar_job} مغتربات",
        "expatriate {en_job}": "{ar_job} مغتربات",
        # emigrants keys
        "female emigrants {en_job}": "{ar_job} مهاجرات",
        "emigrants female {en_job}": "{ar_job} مهاجرات",
        "emigrants {en_job}": "{ar_job} مهاجرات",
    }

    formatted_data_jobs.update(nat_and_gender_keys("{en_job}", "expatriate", "female", "{ar_job} مغتربات"))
    formatted_data_jobs.update(nat_and_gender_keys("{en_job}", "emigrants", "female", "{ar_job} مهاجرات"))

    formatted_data = dict(formatted_data_jobs)
    formatted_data.update(
        {
            f"{{en_nat}} {x}": f"{v} {{females}}"
            for x, v in formatted_data_jobs.items()
            if "{en_nat}" not in x and "{females}" not in v
        }
    )

    formatted_data.update(
        {f"{{en_nat}}-american {x}": f"{v} أمريكيات {{females}}" for x, v in formatted_data_jobs.items()}
    )

    formatted_data_womens_jobs = {}

    for x, v in genders_keys.items():
        keys_more = one_Keys_more_2(
            x,
            v,
            en_nat_key="{en_nat}",
            en_job_key="{en_job}",
            ar_nat_key="{females}",
            ar_job_key="{ar_job}",
            women_key="female",
            add_women=True,
        )
        formatted_data.update(keys_more)
        formatted_data_womens_jobs.update(
            one_Keys_more_2(
                x,
                v,
                en_nat_key="{en_nat}",
                en_job_key="{en_job}",
                ar_nat_key="{females}",
                ar_job_key="{ar_job}",
                add_women=False,
            )
        )

    formatted_data.update(formatted_data_jobs_with_nat)

    # formatted_data.update({ "{en_nat} female film directors": "مخرجات أفلام {females}"})
    formatted_data.update(
        {
            "{en_nat} female abolitionists": "{females} مناهضات للعبودية",
        }
    )
    formatted_data_final = {x.replace("'", ""): v for x, v in formatted_data.items()}

    formatted_data_womens_jobs = {
        x.replace("'", ""): v for x, v in formatted_data_womens_jobs.items() if "{en_nat}" not in x
    }

    return formatted_data_final, formatted_data_womens_jobs


@functools.lru_cache(maxsize=1)
def _load_jobs_data() -> dict[str, str]:
    # all keys without any word from not_in_keys
    data = {x: {"ar_job": v} for x, v in jobs_womens_data.items() if not is_false_key(x, v)}
    len_diff = len(set(jobs_womens_data.keys()) - set(data.keys()))
    if len_diff:
        logger.error(f"_load_jobs_data womens before fix: {len(data):,}, is_false_key diff: {len_diff:,}")

    # data.update({x: {"ar_job": v} for x, v in FEMALE_JOBS_BASE_EXTENDED.items()})

    data = {x.replace("'", "").replace("australian rules", "australian-rules"): v for x, v in data.items()}
    return data


@functools.lru_cache(maxsize=1)
def _load_nat_data() -> dict[str, str]:
    # nats_data: dict[str, str] = {x: v for x, v in all_country_with_nat_ar.items()}  # 342
    nats_data: dict[str, str] = {x: v for x, v in All_Nat.items()}  # 342

    nats_data.update({x: v for x, v in nats_keys_as_country_names.items()})

    nats_data.update(
        {
            "jewish american": {
                "male": "أمريكي يهودي",
                "males": "أمريكيون يهود",
                "female": "أمريكية يهودية",
                "females": "أمريكيات يهوديات",
                "the_male": "الأمريكي اليهودي",
                "the_female": "الأمريكية اليهودية",
                "en": "",
                "ar": "",
            }
        }
    )

    nats_data = {x.replace("'", ""): v for x, v in nats_data.items()}
    return nats_data


@functools.lru_cache(maxsize=1)
def load_bot() -> MultiDataFormatterBaseV2:
    jobs_data_enhanced = _load_jobs_data()
    logger.debug(f"jobs_data_enhanced womens: {len(jobs_data_enhanced):,}")

    formatted_data, _ = _load_formatted_data()

    logger.debug(f"_load_formatted_data formatted_data: {len(formatted_data):,}")

    nats_data = _load_nat_data()
    return format_multi_data_v2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        # value_placeholder="{females}",
        data_list2=jobs_data_enhanced,
        key2_placeholder="{en_job}",
        # value2_placeholder="{ar_job}",
        text_after=" people",
        text_before="the ",
        use_other_formatted_data=True,
        search_first_part=True,
    )


@functools.lru_cache(maxsize=1)
def load_bot_only_womens() -> MultiDataFormatterBaseV2:
    jobs_data = {x: {"ar_job": v} for x, v in FEMALE_JOBS_BASE_EXTENDED.items()}
    logger.debug(f"load_bot_only_womens: {len(jobs_data):,}")

    formatted_data, formatted_data_womens_jobs = _load_formatted_data()
    logger.debug(f"_load_formatted_data formatted_data_womens_jobs: {len(formatted_data_womens_jobs):,}")

    formatted_data_womens_jobs.update(formatted_data)

    nats_data = _load_nat_data()
    return format_multi_data_v2(
        formatted_data=formatted_data_womens_jobs,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        # value_placeholder="{females}",
        data_list2=jobs_data,
        key2_placeholder="{en_job}",
        # value2_placeholder="{ar_job}",
        text_after=" people",
        text_before="the ",
        use_other_formatted_data=True,
        search_first_part=True,
    )


@functools.lru_cache(maxsize=10000)
def _womens_resolver(category: str) -> str:
    logger.debug(f"<<yellow>> start _womens_resolver: {category=}")
    category = fix_keys(category).replace("australian rules", "australian-rules")

    result = load_bot().search_all_category(category)

    logger.info_if_or_debug(f"<<yellow>> end _womens_resolver: {category=}, {result=}", result)
    return result


@functools.lru_cache(maxsize=10000)
def _womens_jobs_resolver(category: str) -> str:
    logger.debug(f"<<yellow>> start _womens_jobs_resolver: {category=}")
    category = fix_keys(category).replace("australian rules", "australian-rules")

    result = load_bot_only_womens().search_all_category(category)

    logger.info_if_or_debug(f"<<yellow>> end _womens_jobs_resolver: {category=}, {result=}", result)
    return result


@functools.lru_cache(maxsize=10000)
def womens_resolver_labels(category: str) -> str:
    # logger.debug(f"<<yellow>> start womens_resolver_labels: {category=}")
    category = fix_keys(category).replace("australian rules", "australian-rules")

    if category in countries_en_as_nationality_keys or category in countries_en_keys:
        logger.info(f"<<yellow>> skip womens_resolver_labels: {category=}, [result=]")
        return ""

    result = _womens_resolver(category) or _womens_jobs_resolver(category)

    # logger.info_if_or_debug(f"<<yellow>> end womens_resolver_labels: {category=}, {result=}", result)
    return result


len_print.data_len(
    "womens.py",
    {
        "womens_formatted_data": _load_formatted_data()[0],
        "formatted_data_womens_jobs": _load_formatted_data()[1],
        "womens_jobs_data_enhanced": _load_jobs_data(),
    },
)



"""
Module to resolve nationality gender patterns in Arabic categories.

"""

import functools
import re

from ..helps import logger
from ..translations import SPORT_KEY_RECORDS_BASE, All_Nat, open_json_file
from ..translations_formats import FormatDataV2, MultiDataFormatterBaseV2, format_multi_data_v2

REGEX_WOMENS = re.compile(r"\b(womens|women)\b", re.I)  # replaced by female
REGEX_MENS = re.compile(r"\b(mens|men)\b", re.I)  # replaced by male


def generate_jobs_data_dict():
    """
    Generate jobs data dictionary.
    """
    # "[ا-ي]+ون* و[ا-ي]+ات"
    jobs_data_new = {
        "bobsledders": {
            "job_males": "متزلجون جماعيون",
            "job_females": "متزلجات جماعيات",
            "both_jobs": "متزلجون ومتزلجات جماعيون",
        },
        "models": {"job_males": "عارضو أزياء", "job_females": "عارضات أزياء", "both_jobs": "عارضو وعارضات أزياء"},
        "actors": {"job_males": "ممثلون", "job_females": "ممثلات", "both_jobs": "ممثلون وممثلات"},
        "boxers": {"job_males": "ملاكمون", "job_females": "ملاكمات", "both_jobs": "ملاكمون وملاكمات"},
        "classical composers": {
            "job_males": "ملحنون كلاسيكيون",
            "job_females": "ملحنات كلاسيكيات",
            "both_jobs": "ملحنون وملحنات كلاسيكيون",
        },
        "composers": {"job_males": "ملحنون", "job_females": "ملحنات", "both_jobs": "ملحنون وملحنات"},
        "cyclists": {"job_males": "دراجون", "job_females": "دراجات", "both_jobs": "دراجون ودراجات"},
        "film actors": {"job_males": "ممثلو أفلام", "job_females": "ممثلات أفلام", "both_jobs": "ممثلو وممثلات أفلام"},
        "guitarists": {
            "job_males": "عازفو قيثارة",
            "job_females": "عازفات قيثارة",
            "both_jobs": "عازفو وعازفات قيثارة",
        },
        "singers": {"job_males": "مغنون", "job_females": "مغنيات", "both_jobs": "مغنون ومغنيات"},
        "television actors": {
            "job_males": "ممثلو تلفاز",
            "job_females": "ممثلات تلفاز",
            "both_jobs": "ممثلو وممثلات تلفاز",
        },
    }
    # TODO: Load from jobs_data_multi_one_word.json
    # jobs_data_new.update(open_json_file("jobs_data_multi_one_word.json"))

    return jobs_data_new


def generate_formatted_data():
    formatted_data = {
        "actresses": "ممثلات",
        "{en_nat} actresses": "ممثلات {females}",
        # [guitarists] = "عازفو وعازفات قيثارة"
        "{job_en}": "{both_jobs}",
        # [male guitarists] = "عازفو قيثارة"
        "male {job_en}": "{job_males}",
        # [american guitarists] = "عازفو وعازفات قيثارة أمريكيون"
        "{en_nat} {job_en}": "{both_jobs} {males}",
        # [american male guitarists] = "عازفو قيثارة أمريكيون"
        "{en_nat} male {job_en}": "{job_males} {males}",
        "male {en_nat} {job_en}": "{job_males} {males}",
        # [female guitarists] = "عازفات قيثارة"
        "female {job_en}": "{job_females}",
        # [american female guitarists] = "عازفات قيثارة أمريكيات"
        "{en_nat} female {job_en}": "{job_females} {females}",
        "female {en_nat} {job_en}": "{job_females} {females}",
        # test to add classical
        "classical {job_en}": "{both_jobs} كلاسيكيون",
        "male classical {job_en}": "{job_males} كلاسيكيون",
        "female classical {job_en}": "{job_females} كلاسيكيات",
        "{en_nat} classical {job_en}": "{both_jobs} كلاسيكيون {males}",
        "{en_nat} male classical {job_en}": "{job_males} كلاسيكيون {males}",
        "{en_nat} female classical {job_en}": "{job_females} كلاسيكيات {females}",
    }

    return formatted_data


@functools.lru_cache(maxsize=1)
def _job_bot() -> MultiDataFormatterBaseV2:
    formatted_data = generate_formatted_data()

    jobs_data_new = generate_jobs_data_dict()

    nats_data = {
        x: {
            "males": v["males"],
            "females": v["females"],
        }
        for x, v in All_Nat.items()
        if v.get("males")
    }

    return format_multi_data_v2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        data_list2=jobs_data_new,
        key2_placeholder="{job_en}",
        text_after="",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )


@functools.lru_cache(maxsize=1)
def _make_sport_formatted_data() -> dict[str, str]:
    formatted_data_new = {
        # _build_players_data - footballers without nats
        "male footballers": "لاعبو كرة قدم",
        "female footballers": "لاعبات كرة قدم",
        "footballers": "لاعبو ولاعبات كرة قدم",
        # _build_players_data - footballers with nats
        "{en_nat} male footballers": "لاعبو كرة قدم {males}",
        "{en_nat} female footballers": "لاعبات كرة قدم {females}",
        "{en_nat} footballers": "لاعبو ولاعبات كرة قدم {males}",
        # _build_players_data - sports without nats
        "female {en_sport} players": "لاعبات {sport_ar}",
        # _build_players_data - sports with nats
        "{en_nat} female {en_sport} players": "لاعبات {sport_ar} {females}",
        # _build_players_data - sports without nats
        "male {en_sport} players": "لاعبو {sport_ar}",
        "{en_sport} players": "لاعبو ولاعبات {sport_ar}",
        # _build_players_data - sports with nats
        "{en_nat} male {en_sport} players": "لاعبو {sport_ar} {males}",
        "{en_nat} {en_sport} players": "لاعبو ولاعبات {sport_ar} {males}",
    }

    players_of_data = {
        "australian rules football": "كرة قدم أسترالية",
        "american-football": "كرة قدم أمريكية",
    }
    for sport, ar in players_of_data.items():
        formatted_data_new.update(
            {
                f"female players of {sport}": f"لاعبات {ar}",
                f"{{en_nat}} female players of {sport}": f"لاعبات {ar} {{females}}",
                f"male players of {sport}": f"لاعبو {ar}",
                f"{{en_nat}} male players of {sport}": f"لاعبو {ar} {{males}}",
                f"players of {sport}": f"لاعبو ولاعبات {ar}",
                f"{{en_nat}} players of {sport}": f"لاعبو ولاعبات {ar} {{males}}",
            }
        )

    return formatted_data_new


@functools.lru_cache(maxsize=1)
def _sport_bot() -> MultiDataFormatterBaseV2:
    sports_data_new = {
        sport: {"sport_ar": record.get("jobs", "")}
        for sport, record in SPORT_KEY_RECORDS_BASE.items()
        if record.get("jobs", "")
    }

    sports_data_new.update(
        {
            "softball": {"sport_ar": "كرة لينة"},
            "futsal": {"sport_ar": "كرة صالات"},
            "badminton": {"sport_ar": "تنس ريشة"},
            "australian rules football": {"sport_ar": "كرة قدم أسترالية"},
            "american-football": {"sport_ar": "كرة قدم أمريكية"},
            "canadian-football": {"sport_ar": "كرة قدم كندية"},
        }
    )

    nats_data = {
        x: {
            "males": v["males"],
            "females": v["females"],
        }
        for x, v in All_Nat.items()
        if v.get("males")
    }
    formatted_data_new = _make_sport_formatted_data()

    return format_multi_data_v2(
        formatted_data=formatted_data_new,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        data_list2=sports_data_new,
        key2_placeholder="{en_sport}",
        text_after="",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )


@functools.lru_cache(maxsize=10000)
def fix_keys(category: str) -> str:
    category = category.lower().replace("category:", "")
    category = category.replace("'", "")

    replacements = {
        "expatriates": "expatriate",
        "canadian football": "canadian-football",
        "american football": "american-football",
    }

    for old, new in replacements.items():
        category = category.replace(old, new)

    category = REGEX_WOMENS.sub("female", category)
    category = REGEX_MENS.sub("male", category)

    return category.strip()


@functools.lru_cache(maxsize=10000)
def genders_jobs_resolver(category: str) -> str:
    normalized_category = fix_keys(category)
    logger.debug(f"<<yellow>> start genders_jobs_resolver: {normalized_category=}")

    job_bot = _job_bot()

    result = job_bot.search_all_other_first(normalized_category)
    if result and category.lower().startswith("category:"):
        result = "تصنيف:" + result
    logger.info_if_or_debug(f"<<yellow>> end genders_jobs_resolver: {category=}, {result=}", result)

    return result


@functools.lru_cache(maxsize=10000)
def genders_sports_resolver(category: str) -> str:
    normalized_category = fix_keys(category)
    logger.debug(f"<<yellow>> start genders_sports_resolver: {normalized_category=}")

    sport_bot = _sport_bot()

    result = sport_bot.search_all_other_first(normalized_category)

    if result and category.lower().startswith("category:"):
        result = "تصنيف:" + result
    logger.info_if_or_debug(f"<<yellow>> end genders_sports_resolver: {category=}, {result=}", result)

    return result


@functools.lru_cache(maxsize=10000)
def resolve_nat_genders_pattern_v2(category: str) -> str:
    logger.debug(f"<<yellow>> start resolve_nat_genders_pattern_v2: {category=}")

    result = genders_sports_resolver(category) or genders_jobs_resolver(category) or ""
    logger.info_if_or_debug(f"<<yellow>> end resolve_nat_genders_pattern_v2: {category=}, {result=}", result)

    return result


__all__ = [
    "resolve_nat_genders_pattern_v2",
]



"""
Cycling specific lookup tables used by the sports translation modules.
"""

from ...helps import len_print

BASE_CYCLING_EVENTS: dict[str, str] = {
    "tour de france": "سباق طواف فرنسا",
    "grand tour": "الطوافات الكبرى",
    "grand tour (cycling)": "الطوافات الكبرى",
    "vuelta a españa": "طواف إسبانيا",
    "giro d'italia": "طواف إيطاليا",
    "presidential cycling tour of turkey": "طواف تركيا",
    "tour de suisse": "طواف سويسرا",
    "vuelta a colombia": "طواف كولومبيا",
    "vuelta a venezuela": "فويلتا فنزويلا",
}


def build_cycling_templates() -> dict[str, str]:
    """Generate derivative keys for cycling tournaments.

    Returns:
        A mapping keyed by lower-case identifiers with Arabic labels covering
        media, squads, and other related variations for every race.
    """

    templates: dict[str, str] = {}
    for english_name, arabic_label in BASE_CYCLING_EVENTS.items():
        normalized_name = english_name.lower()
        templates[normalized_name] = arabic_label
        templates[f"{normalized_name} media"] = f"إعلام {arabic_label}"
        templates[f"{normalized_name} squads"] = f"تشكيلات {arabic_label}"
        templates[f"{normalized_name} cyclists"] = f"دراجو {arabic_label}"
        templates[f"{normalized_name} directors"] = f"مدراء {arabic_label}"
        templates[f"{normalized_name} journalists"] = f"صحفيو {arabic_label}"
        templates[f"{normalized_name} people"] = f"أعلام {arabic_label}"
        templates[f"{normalized_name} stages"] = f"مراحل {arabic_label}"
        templates[f"{normalized_name} stage winners"] = f"فائزون في مراحل {arabic_label}"
    return templates


CYCLING_TEMPLATES: dict[str, str] = build_cycling_templates()

__all__ = [
    "BASE_CYCLING_EVENTS",
    "CYCLING_TEMPLATES",
    "build_cycling_templates",
]

len_print.data_len(
    "cycling.py",
    {
        "CYCLING_TEMPLATES": CYCLING_TEMPLATES,
    },
)



#
import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_arabic_category_label

empty_data = {
    "Category:Abandoned military aircraft projects of Russia": "",
    "Category:Abandoned military projects of Russia": "",
    "Category:Academicians of the National Academy of Motion Picture Arts and Sciences of Russia": "",
    "Category:Academic staff of the D. Mendeleev University of Chemical Technology of Russia": "",
    "Category:Academic staff of the Peoples' Friendship University of Russia": "",
    "Category:Academies of Russia": "",
    "Category:Acting heads of the federal subjects of Russia": "",
    "Category:Administrative divisions of Russia": "",
    "Category:Administrative divisions of the federal subjects of Russia": "",
    "Category:Advisers to the president of Russia": "",
    "Category:Aerial bombs of Russia": "",
    "Category:Aerospace companieszz of Russia": "",
    "Category:Agricultural machinery manufacturers of Russia": "",
    "Category:Aides to the president of Russia": "",
    "Category:Airborne divisions of Russia": "",
    "Category:Airborne infantry brigades of Russia": "",
    "Category:Airborne units and formations of Russia": "",
    "Category:Aircraft engine manufacturers of Russia": "",
    "Category:Aircraft industry of Russia": "",
    "Category:Aircraft manufacturers of Russia": "",
    "Category:Air guns of Russia": "",
    "Category:Airport operators of Russia": "",
    "Category:Airships of Russia": "",
    "Category:Air-to-air missile manufacturers of Russia": "",
    "Category:Air-to-air missiles of Russia": "",
    "Category:Air-to-surface missiles of Russia": "",
    "Category:Alexis of Russia": "",
    "Category:Amendments to the Constitution of Russia": "",
    "Category:Ancient peoples of Russia": "",
    "Category:Anna of Russia": "",
    "Category:Anti-aircraft weapons of Russia": "",
    "Category:Anti-ballistic missiles of Russia": "",
    "Category:Anti-ship cruise missiles of Russia": "",
    "Category:Anti-ship missiles of Russia": "",
    "Category:Anti-submarine missiles of Russia": "",
    "Category:Anti-tank guided missiles of Russia": "",
    "Category:Armoured fighting vehicles of Russia": "",
    "Category:Armoured personnel carriers of Russia": "",
    "Category:Armoured units and formations of Russia": "",
    "Category:Arms industry of Russia": "",
    "Category:Arms trading companieszz of Russia": "",
    "Category:Army training units and formations of Russia": "",
    "Category:Army units and formations of Russia": "",
    "Category:Artillery units and formations of Russia": "",
    "Category:Assault rifles of Russia": "",
    "Category:Audio equipment manufacturers of Russia": "",
    "Category:Autonomous okrugs of Russia": "",
    "Category:Auto parts suppliers of Russia": "",
    "Category:Aviation history of Russia": "",
    "Category:Ballistic missiles of Russia": "",
    "Category:Barges of Russia": "",
    "Category:Battalions of Russia": "",
    "Category:Bilateral sports relations of Russia": "",
    "Category:Bilateral treaties of Russia": "",
    "Category:Biosphere reserves of Russia": "",
    "Category:Bishops of the Russian Orthodox Church Outside of Russia": "",
    "Category:Bolt-action rifles of Russia": "",
    "Category:Book publishing companieszz of Russia": "",
    "Category:Books by publishing company of Russia": "",
    "Category:Bookstores of Russia": "",
    "Category:Boycotts of Russia": "",
    "Category:Brigades of Russia": "",
    "Category:Bullion coins of Russia": "",
    "Category:Buses of Russia": "",
    "Category:Cable television companieszz of Russia": "",
    "Category:Calderas of Russia": "",
    "Category:Capitals of republics of Russia": "",
    "Category:Car manufacturers of Russia": "",
    "Category:Categories by federal subject of Russia": "",
    "Category:Central Bank of Russia": "",
    "Category:Ceramics manufacturers of Russia": "",
    "Category:Chief Muftis of Russia": "",
    "Category:Chief rabbis of Russia": "",
    "Category:Circassian people of Russia": "",
    "Category:City districts of Russia": "",
    "Category:Client states of Russia": "",
    "Category:Climbing areas of Russia": "",
    "Category:Clothing brands of Russia": "",
    "Category:Coins of Russia": "",
    "Category:Combat helmets of Russia": "",
    "Category:Comic book publishing companieszz of Russia": "",
    "Category:Communications satellites of Russia": "",
    "Category:Concert tours of Russia": "",
    "Category:Constitutional history of Russia": "",
    "Category:Constitutions and charters of federal subjects of Russia": "",
    "Category:Construction equipment manufacturers of Russia": "",
    "Category:Continental shelves of Russia": "",
    "Category:Copper mining companieszz of Russia": "",
    "Category:Corps of Russia": "",
    "Category:Country data templates of former subdivisions of Russia": "",
    "Category:Country data templates of Russia": "",
    "Category:Country data templates of subdivisions of Russia": "",
    "Category:Court of Elizabeth of Russia": "",
    "Category:Crewed space program of Russia": "",
    "Category:Cruise missiles of Russia": "",
    "Category:Cultural heritage of Russia": "",
    "Category:Defunct aircraft manufacturers of Russia": "",
    "Category:Defunct diplomatic missions of Russia": "",
    "Category:Defunct federal ministries of Russia": "",
    "Category:Defunct government agencies of Russia": "",
    "Category:Defunct law enforcement agencies of Russia": "",
    "Category:Defunct motorcycle manufacturers of Russia": "",
    "Category:Department stores of Russia": "",
    "Category:Deputy defence ministers of Russia": "",
    "Category:Deputy foreign ministers of Russia": "",
    "Category:Deputy ministers of Russia": "",
    "Category:Diatremes of Russia": "",
    "Category:Diesel–electric locomotives of Russia": "",
    "Category:Diesel locomotives of Russia": "",
    "Category:Diesel multiple units of Russia": "",
    "Category:Divisions of Russia": "",
    "Category:D. Mendeleev University of Chemical Technology of Russia": "",
    "Category:Double-barreled shotguns of Russia": "",
    "Category:Earth observation satellites of Russia": "",
    "Category:Economic regions of Russia": "",
    "Category:Electric locomotives of Russia": "",
    "Category:Electric multiple units of Russia": "",
    "Category:Electric vehicle manufacturers of Russia": "",
    "Category:Emperors of Russia": "",
    "Category:Empresses consort of Russia": "",
    "Category:Empresses of Russia": "",
    "Category:Empresses regnant of Russia": "",
    "Category:Endemic birds of Russia": "",
    "Category:Endemic fauna of Russia": "",
    "Category:Endemic flora of Russia": "",
    "Category:Energy policy of Russia": "",
    "Category:Engine manufacturers of Russia": "",
    "Category:Estuaries of Russia": "",
    "Category:Eurovision songs of Russia": "",
    "Category:Event management companieszz of Russia": "",
    "Category:Extinct animals of Russia": "",
    "Category:Extreme points of Russia": "",
    "Category:Federal cities of Russia": "",
    "Category:Federal districts of Russia": "",
    "Category:Federal executive bodies of Russia": "",
    "Category:Federal law enforcement agencies of Russia": "",
    "Category:Federal ministries of Russia": "",
    "Category:Federal State Unitary Enterprises of Russia": "",
    "Category:Ferns of Russia": "",
    "Category:Ferry companieszz of Russia": "",
    "Category:Fertilizer companieszz of Russia": "",
    "Category:Field marshals of Russia": "",
    "Category:Film production companieszz of Russia": "",
    "Category:Financial regulatory authorities of Russia": "",
    "Category:Firearm manufacturers of Russia": "",
    "Category:Firearms of Russia": "",
    "Category:Fjords of Russia": "",
    "Category:Food provision ministers of Russia": "",
    "Category:Food retailers of Russia": "",
    "Category:Foreign trade of Russia": "",
    "Category:Former administrative units of Russia": "",
    "Category:Former federal subjects of Russia": "",
    "Category:Former islands of Russia": "",
    "Category:Former subdivisions of Russia": "",
    "Category:Fossils of Russia": "",
    "Category:Geologic formations of Russia": "",
    "Category:Geologic history of Russia": "",
    "Category:Geysers of Russia": "",
    "Category:Glacial lakes of Russia": "",
    "Category:Gold mining companieszz of Russia": "",
    "Category:Government-owned banks of Russia": "",
    "Category:Government spokespersons of Russia": "",
    "Category:Governors of the Central Bank of Russia": "",
    "Category:Grand duchesses of Russia": "",
    "Category:Grand dukes of Russia": "",
    "Category:Grasses of Russia": "",
    "Category:Grenade launchers of Russia": "",
    "Category:Guided missiles of Russia": "",
    "Category:Heads of former federal subjects of Russia": "",
    "Category:Heavy machine guns of Russia": "",
    "Category:Helicopter manufacturers of Russia": "",
    "Category:Helicopter regiments of Russia": "",
    "Category:High-speed trains of Russia": "",
    "Category:Historical ethnic groups of Russia": "",
    "Category:Historical geography of Russia": "",
    "Category:History of the administrative divisions of Russia": "",
    "Category:Home appliance manufacturers of Russia": "",
    "Category:Honorary titles of Russia": "",
    "Category:Howitzers of Russia": "",
    "Category:Hydroelectric power companieszz of Russia": "",
    "Category:Icebreakers of Russia": "",
    "Category:Ice caps of Russia": "",
    "Category:Identified objects of cultural heritage of Russia": "",
    "Category:Impact craters of Russia": "",
    "Category:Important Bird Areas of Russia": "",
    "Category:Independent Psychiatric Association of Russia": "",
    "Category:Indigenous languages of Russia": "",
    "Category:Infantry brigades of Russia": "",
    "Category:Infantry divisions of Russia": "",
    "Category:Infantry fighting vehicles of Russia": "",
    "Category:Infantry units and formations of Russia": "",
    "Category:Information technology consulting firms of Russia": "",
    "Category:Intercontinental ballistic missiles of Russia": "",
    "Category:Intermediate-range ballistic missiles of Russia": "",
    "Category:Internet technology companieszz of Russia": "",
    "Category:Investment management companieszz of Russia": "",
    "Category:Isthmuses of Russia": "",
    "Category:Judges of the Constitutional Court of Russia": "",
    "Category:Krais of Russia": "",
    "Category:Krivak-class frigates of the Border Guard Service of Russia": "",
    "Category:Lagoons of Russia": "",
    "Category:Lake groups of Russia": "",
    "Category:Lake islands of Russia": "",
    "Category:Land cultivation ministers of Russia": "",
    "Category:Law enforcement agencies of the republics of Russia": "",
    "Category:Liberal Democratic Party of Russia": "",
    "Category:Light tanks of Russia": "",
    "Category:Liquefied natural gas plants of Russia": "",
    "Category:Lists of commemorative coins of Russia": "",
    "Category:Lists of moths of Russia": "",
    "Category:Lithostratigraphy of Russia": "",
    "Category:Locomotives of Russia": "",
    "Category:Lost submarines of Russia": "",
    "Category:Machine guns of Russia": "",
    "Category:Main battle tanks of Russia": "",
    "Category:Maps of Russia": "",
    "Category:Mechanical engineering companieszz of Russia": "",
    "Category:Mechanised infantry brigades of Russia": "",
    "Category:Medical technology companieszz of Russia": "",
    "Category:Medieval history of Russia": "",
    "Category:Members of the Supreme Soviet of Russia": "",
    "Category:Merchant ships of Russia": "",
    "Category:Metallurgical industry of Russia": "",
    "Category:Metropolitan areas of Russia": "",
    "Category:Military aircraft procurement programs of Russia": "",
    "Category:Military awards and decorations of Russia": "",
    "Category:Military electronics of Russia": "",
    "Category:Military locations of Russia": "",
    "Category:Military satellites of Russia": "",
    "Category:Military vehicles of Russia": "",
    "Category:Model manufacturers of Russia": "",
    "Category:Modern anti-tank rockets of Russia": "",
    "Category:Modern incendiary weapons of Russia": "",
    "Category:Modern military equipment of Russia": "",
    "Category:Modern thermobaric weapons of Russia": "",
    "Category:Monasteries of the Russian Orthodox Church Outside of Russia": "",
    "Category:Monitors of Russia": "",
    "Category:Motorcycle manufacturers of Russia": "",
    "Category:Motorcycles of Russia": "",
    "Category:Multilateral relations of Russia": "",
    "Category:Multiple units of Russia": "",
    "Category:Municipal districts of Russia": "",
    "Category:Municipal divisions of the federal subjects of Russia": "",
    "Category:Museums-Reserves of Russia": "",
    "Category:Musical instrument manufacturing companieszz of Russia": "",
    "Category:Named passenger trains of Russia": "",
    "Category:Nanotechnology companieszz of Russia": "",
    "Category:National orchestras of Russia": "",
    "Category:Natural monuments of Russia": "",
    "Category:Natural Resources and environment ministers of Russia": "",
    "Category:Naval guns of Russia": "",
    "Category:Naval infantry brigades of Russia": "",
    "Category:Naval weapons of Russia": "",
    "Category:Navigation satellites of Russia": "",
    "Category:Nicholas I of Russia": "",
    "Category:Night trains of Russia": "",
    "Category:Noble titles of Russia": "",
    "Category:Non-lethal firearms of Russia": "",
    "Category:Nuclear cruise missiles of Russia": "",
    "Category:Nuclear power companieszz of Russia": "",
    "Category:Nuclear technology companieszz of Russia": "",
    "Category:Nuclear weapons program of Russia": "",
    "Category:Oblasts of Russia": "",
    "Category:Online companieszz of Russia": "",
    "Category:Online dating services of Russia": "",
    "Category:Online financial services companieszz of Russia": "",
    "Category:Online gambling companieszz of Russia": "",
    "Category:Online marketplaces of Russia": "",
    "Category:Online music stores of Russia": "",
    "Category:Online retailers of Russia": "",
    "Category:Optics manufacturing companieszz of Russia": "",
    "Category:Orchids of Russia": "",
    "Category:Orders of chivalry of Russia": "",
    "Category:Pacific Coast of Russia": "",
    "Category:Parties affiliated with the Liberal Democratic Party of Russia": "",
    "Category:People's Artists of Russia": "",
    "Category:Peoples' Friendship University of Russia": "",
    "Category:Philately of Russia": "",
    "Category:Photography companieszz of Russia": "",
    "Category:Photography equipment manufacturers of Russia": "",
    "Category:Pistols of Russia": "",
    "Category:Political divisions of Russia": "",
    "Category:Political families of Russia": "",
    "Category:Postage stamps of Russia": "",
    "Category:Postal history of Russia": "",
    "Category:Postal system of Russia": "",
    "Category:Post–Cold War weapons of Russia": "",
    "Category:Presidential Administration of Russia": "",
    "Category:Presidents of the Constitutional Court of Russia": "",
    "Category:Presidents of the Handball Union of Russia": "",
    "Category:Priests of the Russian Orthodox Church Outside of Russia": "",
    "Category:Private equity firms of Russia": "",
    "Category:Proposed aircraft of Russia": "",
    "Category:Proposed military aircraft of Russia": "",
    "Category:Prosecutors-general of Russia": "",
    "Category:Railcars of Russia": "",
    "Category:Rail transport navigational boxes of Russia": "",
    "Category:Rail vehicle manufacturers of Russia": "",
    "Category:Railway coaches of Russia": "",
    "Category:Reconnaissance satellites of Russia": "",
    "Category:Regalia of Russia": "",
    "Category:Regents of Russia": "",
    "Category:Republics of Russia": "",
    "Category:Research vessels of Russia": "",
    "Category:Revolvers of Russia": "",
    "Category:Rifles of Russia": "",
    "Category:River islands of Russia": "",
    "Category:Road-inaccessible communities of Russia": "",
    "Category:Robots of Russia": "",
    "Category:Rocket divisions of Russia": "",
    "Category:Rocket engine manufacturers of Russia": "",
    "Category:Rocket-propelled grenade launchers of Russia": "",
    "Category:Rural settlements of Russia": "",
    "Category:Russian Orthodox Church Outside of Russia": "",
    "Category:Russo-Japanese War battleships of Russia": "",
    "Category:Russo-Japanese War cruisers of Russia": "",
    "Category:Russo-Japanese War naval ships of Russia": "",
    "Category:Russo-Japanese war weapons of Russia": "",
    "Category:Sacred lakes of Russia": "",
    "Category:Sberbank of Russia": "",
    "Category:Sedimentary basins of Russia": "",
    "Category:Self-propelled anti-aircraft weapons of Russia": "",
    "Category:Self-propelled artillery of Russia": "",
    "Category:Semi-automatic pistols of Russia": "",
    "Category:Semi-automatic rifles of Russia": "",
    "Category:Semi-automatic shotguns of Russia": "",
    "Category:Semiconductor companieszz of Russia": "",
    "Category:Shield volcanoes of Russia": "",
    "Category:Ships of the Border Guard Service of Russia": "",
    "Category:Ships of the line of Russia": "",
    "Category:Shotguns of Russia": "",
    "Category:Single-shot shotguns of Russia": "",
    "Category:Sniper rifles of Russia": "",
    "Category:Sounding rockets of Russia": "",
    "Category:Space industry companieszz of Russia": "",
    "Category:Space launch vehicles of Russia": "",
    "Category:Space program of Russia": "",
    "Category:Special Economic Zones of Russia": "",
    "Category:Special envoys of the president of Russia": "",
    "Category:Special forces of Russia": "",
    "Category:Specialist law enforcement agencies of Russia": "",
    "Category:Specialist police agencies of Russia": "",
    "Category:Spetsnaz brigades of Russia": "",
    "Category:Spits of Russia": "",
    "Category:Sports titles of Russia": "",
    "Category:Standards of Russia": "",
    "Category:State controllers of Russia": "",
    "Category:Steam locomotives of Russia": "",
    "Category:Steamships of Russia": "",
    "Category:Steel companieszz of Russia": "",
    "Category:Submachine guns of Russia": "",
    "Category:Submarine-launched ballistic missiles of Russia": "",
    "Category:Submarine-launched cruise missiles of Russia": "",
    "Category:Supreme Soviet of Russia": "",
    "Category:Surface-to-air missiles of Russia": "",
    "Category:Surface-to-surface missiles of Russia": "",
    "Category:Tank brigades of Russia": "",
    "Category:Tank divisions of Russia": "",
    "Category:Tankers of Russia": "",
    "Category:Tank regiments of Russia": "",
    "Category:Tatar people of Russia": "",
    "Category:Telecommunications companieszz of Russia": "",
    "Category:Television production companieszz of Russia": "",
    "Category:Templates by federal subject of Russia": "",
    "Category:Territorial disputes of Russia": "",
    "Category:Textile arts of Russia": "",
    "Category:Theatre rocket brigades of Russia": "",
    "Category:Third-level administrative divisions of Russia": "",
    "Category:Toy companieszz of Russia": "",
    "Category:Tractor manufacturers of Russia": "",
    "Category:Trading companieszz of Russia": "",
    "Category:Tram vehicles of Russia": "",
    "Category:Trees of Russia": "",
    "Category:Trial and research firearms of Russia": "",
    "Category:Tsareviches of Russia": "",
    "Category:Tsarevnas of Russia": "",
    "Category:Tsarinas of Russia": "",
    "Category:Tsars of Russia": "",
    "Category:Tsesarevichs of Russia": "",
    "Category:Unitary Enterprises of Russia": "",
    "Category:Units and formations of the National Guard of Russia": "",
    "Category:Uranium mining companieszz of Russia": "",
    "Category:Urban okrugs of Russia": "",
    "Category:Urban settlements of Russia": "",
    "Category:Vasili IV of Russia": "",
    "Category:Venture capital firms of Russia": "",
    "Category:Volcanism of Russia": "",
    "Category:Watch manufacturing companieszz of Russia": "",
    "Category:Weapons of Russia": "",
    "Category:Weather satellites of Russia": "",
    "Category:Wikipedia categories named after companieszz of Russia": "",
    "Category:Wikipedia categories named after manufacturing companieszz of Russia": "",
    "Category:Wikipedia categories named after presidents of Russia": "",
    "Category:Wikipedia featured topics Battlecruisers of Russia": "",
    "Category:Wikipedia requested images of people of Russia": "",
    "Category:Wikipedias by languages of Russia": "",
    "Category:Wine regions of Russia": "",
    "Category:Wineries of Russia": "",
    "Category:Women heads of federal subjects of Russia": "",
    "Category:Wooden architecture of Russia": "",
    "Category:World War I armoured fighting vehicles of Russia": "",
    "Category:World War I artillery of Russia": "",
    "Category:World War I battlecruisers of Russia": "",
    "Category:World War I battleships of Russia": "",
    "Category:World War I cruisers of Russia": "",
    "Category:World War I destroyers of Russia": "",
    "Category:World War I merchant ships of Russia": "",
    "Category:World War I military equipment of Russia": "",
    "Category:World War I mortars of Russia": "",
    "Category:World War I naval ships of Russia": "",
    "Category:World War I passenger ships of Russia": "",
    "Category:World War I ships of Russia": "",
    "Category:World War I submarines of Russia": "",
    "Category:World War I tanks of Russia": "",
    "Category:World War I torpedo boats of Russia": "",
    "Category:World War I weapons of Russia": "",
}

data1 = {
    "Category:Agriculture ministers of Russia": "تصنيف:وزراء زراعة روسيا",
    "Category:Culture ministers of Russia": "تصنيف:وزراء ثقافة روسيا",
    "Category:Economy ministers of Russia": "تصنيف:وزراء اقتصاد روسيا",
    "Category:Education ministers of Russia": "تصنيف:وزراء تعليم روسيا",
    "Category:Energy ministers of Russia": "تصنيف:وزراء طاقة روسيا",
    "Category:Finance ministers of Russia": "تصنيف:وزراء مالية روسيا",
    "Category:Health ministers of Russia": "تصنيف:وزراء صحة روسيا",
    "Category:Justice ministers of Russia": "تصنيف:وزراء عدل روسيا",
    "Category:Military personnel of the Tsardom of Russia": "تصنيف:أفراد عسكريون من روسيا القيصرية",
    "Category:Immigrants to the Tsardom of Russia": "تصنيف:مهاجرون إلى روسيا القيصرية",
    "Category:Generals of the Tsardom of Russia": "تصنيف:جنرالات روسيا القيصرية",
    "Category:Foreign relations of the Tsardom of Russia": "تصنيف:علاقات روسيا القيصرية الخارجية",
    "Category:Foreign ministers of the Tsardom of Russia": "تصنيف:وزراء خارجية روسيا القيصرية",
    "Category:Battles involving the Tsardom of Russia": "تصنيف:معارك تشمل روسيا القيصرية",
    "Category:Battles of the French invasion of Russia": "تصنيف:معارك الغزو الفرنسي لروسيا",
    "Category:Rebels from the Tsardom of Russia": "تصنيف:متمردون من روسيا القيصرية",
    "Category:People from the Tsardom of Russia": "تصنيف:أشخاص من روسيا القيصرية",
    "Category:Nuns from the Tsardom of Russia": "تصنيف:راهبات من روسيا القيصرية",
    "Category:Nobility from the Tsardom of Russia": "تصنيف:نبلاء من روسيا القيصرية",
    "Category:English expatriates in the Tsardom of Russia": "تصنيف:إنجليز مغتربون في روسيا القيصرية",
    "Category:Expatriates in the Tsardom of Russia": "تصنيف:مغتربون في روسيا القيصرية",
    "Category:Explorers from the Tsardom of Russia": "تصنيف:مستكشفون من روسيا القيصرية",
    "Category:Expatriates from the Dutch Republic in the Tsardom of Russia": "تصنيف:مغتربون من جمهورية هولندا في روسيا القيصرية",
    "Category:Artists from the Tsardom of Russia": "تصنيف:فنانون من روسيا القيصرية",
    "Category:Writers from the Tsardom of Russia": "تصنيف:كتاب من روسيا القيصرية",
    "Category:Ambassadors of Safavid Iran to the Tsardom of Russia": "تصنيف:سفراء إيران الصفوية لدى روسيا القيصرية",
    "Category:Ambassadors of Sweden to the Tsardom of Russia": "تصنيف:سفراء السويد لدى روسيا القيصرية",
    "Category:Ambassadors of the Tsardom of Russia": "تصنيف:سفراء روسيا القيصرية",
    "Category:Ambassadors to the Tsardom of Russia": "تصنيف:سفراء لدى روسيا القيصرية",
    "Category:15th-century military history of Russia": "تصنيف:تاريخ روسيا العسكري في القرن 15",
    "Category:16th-century military history of Russia": "تصنيف:تاريخ روسيا العسكري في القرن 16",
    "Category:17th-century military history of Russia": "تصنيف:تاريخ روسيا العسكري في القرن 17",
    "Category:20th-century military history of Russia": "تصنيف:تاريخ روسيا العسكري في القرن 20",
    "Category:21st-century military history of Russia": "تصنيف:تاريخ روسيا العسكري في القرن 21",
    "Category:Accidents and incidents by airline of Russia": "تصنيف:حوادث حسب شركة الطيران في روسيا",
    "Category:Acting presidents of Russia": "تصنيف:رؤساء روسيا بالإنابة",
    "Category:Aircraft carriers of Russia": "تصنيف:حاملات طائرات في روسيا",
    "Category:Air defence units and formations of Russia": "تصنيف:وحدات وتشكيلات دفاع جوي في روسيا",
    "Category:Airlines of Russia": "تصنيف:شركات طيران في روسيا",
    "Category:Ambassadors of Russia": "تصنيف:سفراء روسيا",
    "Category:Amphibians of Russia": "تصنيف:برمائيات روسيا",
    "Category:Amphibious warfare vessels of Russia": "تصنيف:سفن حربية برمائية في روسيا",
    "Category:Archaeology of Russia": "تصنيف:علم الآثار في روسيا",
    "Category:Archipelagoes of Russia": "تصنيف:أرخبيلات روسيا",
    "Category:Architecture firms of Russia": "تصنيف:شركات هندسة معمارية في روسيا",
    "Category:Arthropods of Russia": "تصنيف:مفصليات روسيا",
    "Category:Artificial islands of Russia": "تصنيف:جزر اصطناعية في روسيا",
    "Category:Artillery of Russia": "تصنيف:مدفعية في روسيا",
    "Category:Auxiliary ships of Russia": "تصنيف:سفن مساعدة في روسيا",
    "Category:Banks of Russia": "تصنيف:بنوك روسيا",
    "Category:Battlecruisers of Russia": "تصنيف:طرادات معركة في روسيا",
    "Category:Battleships of Russia": "تصنيف:بوارج في روسيا",
    "Category:Bays of Russia": "تصنيف:خلجان روسيا",
    "Category:Beaches of Russia": "تصنيف:شواطئ روسيا",
    "Category:Bibliographies of Russia": "تصنيف:ببليوجرافيات روسيا",
    "Category:Bilateral military relations of Russia": "تصنيف:علاقات روسيا الثنائية العسكرية",
    "Category:Bilateral relations of Russia": "تصنيف:علاقات روسيا الثنائية",
    "Category:Biota of Russia": "تصنيف:حيويات روسيا",
    "Category:Birds of Russia": "تصنيف:طيور روسيا",
    "Category:Bodies of ice of Russia": "تصنيف:هيئات جليد في روسيا",
    "Category:Bodies of water of Russia": "تصنيف:مسطحات مائية في روسيا",
    "Category:Border crossings of Russia": "تصنيف:معابر حدودية في روسيا",
    "Category:Borders of Russia": "تصنيف:حدود روسيا",
    "Category:Bus manufacturers of Russia": "تصنيف:مصانع الباصات في روسيا",
    "Category:Candidates for President of Russia": "تصنيف:مرشحون لمنصب رئيس روسيا",
    "Category:Canyons and gorges of Russia": "تصنيف:أخاديد في روسيا",
    "Category:Cargo airlines of Russia": "تصنيف:شحن جوي في روسيا",
    "Category:Cargo ships of Russia": "تصنيف:سفن بضائع روسيا",
    "Category:Cars of Russia": "تصنيف:سيارات في روسيا",
    "Category:Caves of Russia": "تصنيف:كهوف روسيا",
    "Category:Charter airlines of Russia": "تصنيف:طيران عارض في روسيا",
    "Category:Chief justices of Russia": "تصنيف:رؤساء قضاء روسيا",
    "Category:Children of presidents of Russia": "تصنيف:أبناء رؤساء روسيا",
    "Category:Children of prime ministers of Russia": "تصنيف:أبناء رؤساء وزراء روسيا",
    "Category:Cinema of Russia": "تصنيف:سينما في روسيا",
    "Category:Citizenship of Russia": "تصنيف:المواطنة في روسيا",
    "Category:Civil awards and decorations of Russia": "تصنيف:جوائز وأوسمة مدنية في روسيا",
    "Category:Climate of Russia": "تصنيف:مناخ روسيا",
    "Category:Coasts of Russia": "تصنيف:سواحل روسيا",
    "Category:Constitutions of Russia": "تصنيف:دساتير في روسيا",
    "Category:Construction industry of Russia": "تصنيف:صناعة بناء في روسيا",
    "Category:Corvettes of Russia": "تصنيف:فرقيطات في روسيا",
    "Category:Cruisers of Russia": "تصنيف:طرادات في روسيا",
    "Category:Crustaceans of Russia": "تصنيف:قشريات روسيا",
    "Category:Cultural depictions of presidents of Russia": "تصنيف:تصوير ثقافي عن رؤساء روسيا",
    "Category:Cultural history of Russia": "تصنيف:تاريخ روسيا الثقافي",
    "Category:Culture of Russia": "تصنيف:ثقافة روسيا",
    "Category:Currencies of Russia": "تصنيف:عملات في روسيا",
    "Category:Defence agencies of Russia": "تصنيف:وكالات دفاع في روسيا",
    "Category:Defunct airlines of Russia": "تصنيف:شركات طيران سابقة في روسيا",
    "Category:Defunct banks of Russia": "تصنيف:بنوك سابقة في روسيا",
    "Category:Defunct motor vehicle manufacturers of Russia": "تصنيف:مصانع سيارات سابقة في روسيا",
    "Category:Demographics of Russia": "تصنيف:التركيبة السكانية في روسيا",
    "Category:Depressions of Russia": "تصنيف:منخفضات روسيا",
    "Category:Deserts of Russia": "تصنيف:صحاري روسيا",
    "Category:Destroyers of Russia": "تصنيف:مدمرات في روسيا",
    "Category:Dinosaurs of Russia": "تصنيف:ديناصورات في روسيا",
    "Category:Diplomatic missions of Russia": "تصنيف:بعثات روسيا الدبلوماسية",
    "Category:Districts of Russia": "تصنيف:مديريات روسيا",
    "Category:Drainage basins of Russia": "تصنيف:أحواض تصريف في روسيا",
    "Category:Drug policy of Russia": "تصنيف:سياسة المخدرات في روسيا",
    "Category:Dunes of Russia": "تصنيف:كثبان رملية في روسيا",
    "Category:Early modern history of Russia": "تصنيف:تاريخ روسيا الحديث المبكر",
    "Category:Economic history of Russia": "تصنيف:تاريخ روسيا الاقتصادي",
    "Category:Economy of Russia": "تصنيف:اقتصاد روسيا",
    "Category:Ecoregions of Russia": "تصنيف:مناطق بيئية في روسيا",
    "Category:Elizabeth of Russia": "تصنيف:إليزابيث إمبراطورة روسيا",
    "Category:Environment of Russia": "تصنيف:بيئة روسيا",
    "Category:Escort ships of Russia": "تصنيف:سفن مرافقة في روسيا",
    "Category:Fast-food chains of Russia": "تصنيف:سلاسل وجبات سريعة في روسيا",
    "Category:Fauna of Russia": "تصنيف:حيوانات روسيا",
    "Category:Featured pictures of Russia": "تصنيف:صور مميزة عن روسيا",
    "Category:Federal subjects of Russia": "تصنيف:كيانات روسيا الاتحادية",
    "Category:Federal universities of Russia": "تصنيف:جامعات فيدرالية في روسيا",
    "Category:Female candidates for President of Russia": "تصنيف:مرشحات لمنصب رئيس روسيا",
    "Category:Ferries of Russia": "تصنيف:عبارات في روسيا",
    "Category:First ladies of Russia": "تصنيف:السيدات الأولى في روسيا",
    "Category:Fish of Russia": "تصنيف:أسماك روسيا",
    "Category:Flags of Russia": "تصنيف:أعلام روسيا",
    "Category:Flags of the federal subjects of Russia": "تصنيف:أعلام كيانات روسيا الاتحادية",
    "Category:Flora of Russia": "تصنيف:نباتات روسيا",
    "Category:Folklore of Russia": "تصنيف:فلكور في روسيا",
    "Category:Forests of Russia": "تصنيف:غابات روسيا",
    "Category:French invasion of Russia": "تصنيف:الغزو الفرنسي لروسيا",
    "Category:Frigates of Russia": "تصنيف:فرقاطات في روسيا",
    "Category:Fungi of Russia": "تصنيف:فطريات روسيا",
    "Category:Geography of Russia": "تصنيف:جغرافيا روسيا",
    "Category:Geology of Russia": "تصنيف:جيولوجيا روسيا",
    "Category:Glaciers of Russia": "تصنيف:أنهار جليدية في روسيا",
    "Category:Golden Ring of Russia": "تصنيف:الجولدن رنج في روسيا",
    "Category:Government agencies of Russia": "تصنيف:وكالات حكومية في روسيا",
    "Category:Government ministers of Russia": "تصنيف:وزراء روسيا",
    "Category:Government ministries of federal subjects of Russia": "تصنيف:وزارات حكومة كيانات روسيا الاتحادية",
    "Category:Government of Russia": "تصنيف:حكومة روسيا",
    "Category:Grand Duchess Anastasia Nikolaevna of Russia": "تصنيف:الدوقة الكبرى أناستاسيا نيكولايفنا من روسيا",
    "Category:Grasslands of Russia": "تصنيف:أراضي عشبية في روسيا",
    "Category:Gulfs of Russia": "تصنيف:خلجان روسيا",
    "Category:Headlands of Russia": "تصنيف:أراضي رأسية في روسيا",
    "Category:Heads of government of Russia": "تصنيف:قادة حكومة روسيا",
    "Category:Heads of state of Russia": "تصنيف:قادة روسيا",
    "Category:Heads of the federal subjects of Russia": "تصنيف:قادة كيانات روسيا الاتحادية",
    "Category:Hills of Russia": "تصنيف:تلال روسيا",
    "Category:Historians of Russia": "تصنيف:مؤرخو روسيا",
    "Category:Historiography of Russia": "تصنيف:تأريخ روسيا",
    "Category:History of Russia": "تصنيف:تاريخ روسيا",
    "Category:Hot springs of Russia": "تصنيف:ينابيع حارة في روسيا",
    "Category:Images of Russia": "تصنيف:صور من روسيا",
    "Category:Indigenous peoples of Russia": "تصنيف:شعوب أصلية في روسيا",
    "Category:Insects of Russia": "تصنيف:حشرات روسيا",
    "Category:Internet service providers of Russia": "تصنيف:تزويد خدمة الإنترنت في روسيا",
    "Category:Invasions of Russia": "تصنيف:غزوات في روسيا",
    "Category:Invertebrates of Russia": "تصنيف:لافقاريات روسيا",
    "Category:Islands of Russia": "تصنيف:جزر روسيا",
    "Category:Judiciary of Russia": "تصنيف:النظام القضائي روسيا",
    "Category:Lakes of Russia": "تصنيف:بحيرات روسيا",
    "Category:Landforms of Russia": "تصنيف:تضاريس روسيا",
    "Category:Languages of Russia": "تصنيف:لغات روسيا",
    "Category:Law enforcement agencies of Russia": "تصنيف:وكالات تطبيق القانون في روسيا",
    "Category:Law of Russia": "تصنيف:قانون في روسيا",
    "Category:Learned societies of Russia": "تصنيف:جمعيات علمية في روسيا",
    "Category:Legal history of Russia": "تصنيف:تاريخ روسيا القانوني",
    "Category:Legislatures of the federal subjects of Russia": "تصنيف:هيئات تشريعية في كيانات روسيا الاتحادية",
    "Category:Lists of ambassadors of Russia": "تصنيف:قوائم سفراء روسيا",
    "Category:Lists of biota of Russia": "تصنيف:قوائم حيويات روسيا",
    "Category:Lists of candidates for President of Russia": "تصنيف:قوائم مرشحون لمنصب رئيس روسيا",
    "Category:Lists of flags of Russia": "تصنيف:قوائم أعلام روسيا",
    "Category:Lists of government ministers of Russia": "تصنيف:قوائم وزراء روسيا",
    "Category:Lists of heads of the federal subjects of Russia": "تصنيف:قوائم قادة كيانات روسيا الاتحادية",
    "Category:Lists of indigenous peoples of Russia": "تصنيف:قوائم شعوب أصلية في روسيا",
    "Category:Lists of landforms of Russia": "تصنيف:قوائم تضاريس روسيا",
    "Category:Lists of ships of Russia": "تصنيف:قوائم سفن روسيا",
    "Category:Lists of subdivisions of Russia": "تصنيف:قوائم تقسيمات روسيا",
    "Category:Locomotive manufacturers of Russia": "تصنيف:مصانع القاطرات في روسيا",
    "Category:Mammals of Russia": "تصنيف:ثدييات روسيا",
    "Category:Maritime history of Russia": "تصنيف:تاريخ روسيا البحري",
    "Category:Military academies of Russia": "تصنيف:أكاديميات عسكرية في روسيا",
    "Category:Military communications of Russia": "تصنيف:اتصالات عسكرية في روسيا",
    "Category:Military equipment of Russia": "تصنيف:عتاد روسيا العسكري",
    "Category:Military history of Russia": "تصنيف:تاريخ روسيا العسكري",
    "Category:Military installations of Russia": "تصنيف:منشآت روسيا العسكرية",
    "Category:Military logistics of Russia": "تصنيف:لوجستية عسكرية في روسيا",
    "Category:Military of Russia": "تصنيف:عسكرية روسيا",
    "Category:Military ranks of Russia": "تصنيف:رتب عسكرية في روسيا",
    "Category:Military research facilities of Russia": "تصنيف:مرافق بحثية عسكرية في روسيا",
    "Category:Military research of Russia": "تصنيف:أبحاث عسكرية في روسيا",
    "Category:Military units and formations of Russia": "تصنيف:وحدات وتشكيلات روسيا العسكرية",
    "Category:Minesweepers of Russia": "تصنيف:كاسحات ألغام في روسيا",
    "Category:Mine warfare vessels of Russia": "تصنيف:سفن حرب ألغام في روسيا",
    "Category:Ministers of defence of Russia": "تصنيف:وزراء دفاع روسيا",
    "Category:Ministers of foreign affairs of Russia": "تصنيف:وزراء شؤون خارجية روسيا",
    "Category:Ministers of internal affairs of Russia": "تصنيف:وزراء شؤون داخلية روسيا",
    "Category:Missile boats of Russia": "تصنيف:قوارب صواريخ في روسيا",
    "Category:Modern history of Russia": "تصنيف:تاريخ روسيا الحديث",
    "Category:Motor vehicle manufacturers of Russia": "تصنيف:مصانع السيارات في روسيا",
    "Category:Mountain passes of Russia": "تصنيف:ممرات جبلية في روسيا",
    "Category:Mountain ranges of Russia": "تصنيف:سلاسل جبلية في روسيا",
    "Category:Mountains of Russia": "تصنيف:جبال روسيا",
    "Category:Municipalities of Russia": "تصنيف:بلديات روسيا",
    "Category:Music of Russia": "تصنيف:موسيقى روسيا",
    "Category:Muslim communities of Russia": "تصنيف:مجتمعات إسلامية في روسيا",
    "Category:National Guard of Russia": "تصنيف:الحرس الوطني في روسيا",
    "Category:National parks of Russia": "تصنيف:متنزهات وطنية في روسيا",
    "Category:National trade union centers of Russia": "تصنيف:مراكز نقابية وطنية في روسيا",
    "Category:Natural history of Russia": "تصنيف:تاريخ روسيا الطبيعي",
    "Category:Naturalized citizens of Russia": "تصنيف:متجنسون في روسيا",
    "Category:Naval history of Russia": "تصنيف:تاريخ روسيا العسكري البحري",
    "Category:Naval ships of Russia": "تصنيف:سفن قوات بحرية في روسيا",
    "Category:Naval units and formations of Russia": "تصنيف:وحدات وتشكيلات روسيا البحرية",
    "Category:Nuclear history of Russia": "تصنيف:تاريخ روسيا النووي",
    "Category:Nuclear missiles of Russia": "تصنيف:صواريخ نووية في روسيا",
    "Category:Nuclear weapons of Russia": "تصنيف:أسلحة نووية في روسيا",
    "Category:Oil fields of Russia": "تصنيف:حقول نفط في روسيا",
    "Category:Orders, decorations, and medals of Russia": "تصنيف:مراتب تشريفية وأوسمة وميداليات روسيا",
    "Category:Passenger ships of Russia": "تصنيف:سفن ركاب في روسيا",
    "Category:Patrol vessels of Russia": "تصنيف:سفن دورية في روسيا",
    "Category:Peace treaties of Russia": "تصنيف:معاهدات سلام روسيا",
    "Category:Peninsulas of Russia": "تصنيف:أشباه جزر في روسيا",
    "Category:People of Russia": "تصنيف:أشخاص من روسيا",
    "Category:Plains of Russia": "تصنيف:سهول روسيا",
    "Category:Plateaus of Russia": "تصنيف:هضاب روسيا",
    "Category:Political history of Russia": "تصنيف:تاريخ روسيا السياسي",
    "Category:Politics of Russia": "تصنيف:سياسة روسيا",
    "Category:Ports and harbours of Russia": "تصنيف:مرافئ وموانئ روسيا",
    "Category:Presidency of Russia": "تصنيف:رئاسة روسيا",
    "Category:Presidents of Russia": "تصنيف:رؤساء روسيا",
    "Category:Prisoners and detainees of Russia": "تصنيف:سجناء ومعتقلون في روسيا",
    "Category:Protected areas of Russia": "تصنيف:مناطق محمية في روسيا",
    "Category:Regions of Russia": "تصنيف:مناطق روسيا",
    "Category:Reptiles of Russia": "تصنيف:زواحف روسيا",
    "Category:Ridges of Russia": "تصنيف:نتوئات جبلية في روسيا",
    "Category:Rivers of Russia": "تصنيف:أنهار روسيا",
    "Category:Rocket engines of Russia": "تصنيف:محركات صواريخ في روسيا",
    "Category:Rock formations of Russia": "تصنيف:تكوينات صخرية في روسيا",
    "Category:Rolling stock of Russia": "تصنيف:القطارات في روسيا",
    "Category:Satellites of Russia": "تصنيف:أقمار صناعية تتبع روسيا",
    "Category:Seas of Russia": "تصنيف:بحار روسيا",
    "Category:Service vessels of Russia": "تصنيف:سفن خدمة في روسيا",
    "Category:Ships of Russia": "تصنيف:سفن روسيا",
    "Category:Shipwrecks of Russia": "تصنيف:حطام سفن في روسيا",
    "Category:Shipyards of Russia": "تصنيف:حوض بناء سفن في روسيا",
    "Category:Social groups of Russia": "تصنيف:مجموعات اجتماعية في روسيا",
    "Category:Social history of Russia": "تصنيف:تاريخ روسيا الاجتماعي",
    "Category:Society of Russia": "تصنيف:مجتمع روسيا",
    "Category:Spiders of Russia": "تصنيف:عناكب في روسيا",
    "Category:Sports organisations of Russia": "تصنيف:منظمات رياضية في روسيا",
    "Category:Sports organizations of Russia": "تصنيف:منظمات رياضية في روسيا",
    "Category:Springs of Russia": "تصنيف:ينابيع روسيا",
    "Category:Straits of Russia": "تصنيف:مضائق روسيا",
    "Category:Stratovolcanoes of Russia": "تصنيف:براكين طبقية في روسيا",
    "Category:Submarines of Russia": "تصنيف:غواصات روسيا",
    "Category:Supermarkets of Russia": "تصنيف:محلات سوبر ماركت في روسيا",
    "Category:Swamps of Russia": "تصنيف:مستنقعات في روسيا",
    "Category:Tall ships of Russia": "تصنيف:سفن طويلة في روسيا",
    "Category:Tanks of Russia": "تصنيف:دبابات روسيا",
    "Category:Territorial evolution of Russia": "تصنيف:الحركة الاستعمارية الروسية",
    "Category:Torpedoes of Russia": "تصنيف:الطوربيدات في روسيا",
    "Category:Training units and formations of Russia": "تصنيف:وحدات وتشكيلات تدريبية في روسيا",
    "Category:Treaties of Russia": "تصنيف:معاهدات روسيا",
    "Category:Truck manufacturers of Russia": "تصنيف:مصانع الشاحنات في روسيا",
    "Category:Uninhabited islands of Russia": "تصنيف:جزر غير مأهولة في روسيا",
    "Category:Unmanned aerial vehicles of Russia": "تصنيف:طائرات بدون طيار روسية",
    "Category:Unmanned military aircraft of Russia": "تصنيف:طائرات عسكرية بدون طيار روسية",
    "Category:Valleys of Russia": "تصنيف:أودية روسيا",
    "Category:Vehicles of Russia": "تصنيف:مركبات روسيا",
    "Category:Vertebrates of Russia": "تصنيف:فقاريات روسيا",
    "Category:Vice presidents of Russia": "تصنيف:نواب رؤساء روسيا",
    "Category:Volcanoes of Russia": "تصنيف:براكين روسيا",
    "Category:Waterfalls of Russia": "تصنيف:شلالات روسيا",
    "Category:Wetlands of Russia": "تصنيف:مناطق رطبة في روسيا",
    "Category:Women government ministers of Russia": "تصنيف:وزيرات روسيات",
    "Category:National youth sports teams of Russia": "تصنيف:منتخبات رياضية وطنية شبابية روسيا",
    "Category:National sports teams of Russia": "تصنيف:منتخبات رياضية وطنية من روسيا",
    "Category:Members of learned societies of Russia": "تصنيف:أعضاء جمعيات علمية في روسيا",
    "Category:Members of legislatures of the federal subjects of Russia": "تصنيف:أعضاء هيئات تشريعية في كيانات روسيا الاتحادية",
    "Category:Men's national sports teams of Russia": "تصنيف:منتخبات رياضية وطنية رجالية روسيا",
    "Category:Women's national sports teams of Russia": "تصنيف:منتخبات رياضية وطنية نسائية روسيا",
    "Category:Works about the French invasion of Russia": "تصنيف:أعمال عن الغزو الفرنسي في روسيا",
    "Category:History books about the Tsardom of Russia": "تصنيف:كتب تاريخ عن روسيا القيصرية",
    "Category:Films about the French invasion of Russia": "تصنيف:أفلام عن الغزو الفرنسي في روسيا",
    "Category:Books about presidents of Russia": "تصنيف:كتب عن رؤساء روسيا",
    "Category:Mass media by language of Russia": "تصنيف:وسائل إعلام حسب لغة روسيا",
    "Category:Categories by language of Russia": "تصنيف:تصنيفات حسب لغة روسيا",
}

companies_data = {
    "Category:Agriculture companies of Russia": "تصنيف:شركات زراعة في روسيا",
    "Category:Aluminium companies of Russia": "تصنيف:شركات ألومنيوم في روسيا",
    "Category:Automotive companies of Russia": "تصنيف:شركات سيارات في روسيا",
    "Category:Biotechnology companies of Russia": "تصنيف:شركات تقانة حيوية في روسيا",
    "Category:Building materials companies of Russia": "تصنيف:شركات مواد بناء في روسيا",
    "Category:Bus companies of Russia": "تصنيف:شركات الباصات في روسيا",
    "Category:Cement companies of Russia": "تصنيف:شركات أسمنت في روسيا",
    "Category:Chemical companies of Russia": "تصنيف:شركات كيميائية في روسيا",
    "Category:Clothing companies of Russia": "تصنيف:شركات ملابس في روسيا",
    "Category:Coal companies of Russia": "تصنيف:شركات فحم في روسيا",
    "Category:companies of Russia": "تصنيف:شركات في روسيا",
    "Category:Computer companies of Russia": "تصنيف:شركات حوسبة في روسيا",
    "Category:Conglomerate companies of Russia": "تصنيف:تكتل شركات في روسيا",
    "Category:Construction and civil engineering companies of Russia": "تصنيف:شركات بناء وهندسة مدنية في روسيا",
    "Category:Cosmetics companies of Russia": "تصنيف:شركات مستحضرات التجميل في روسيا",
    "Category:Defence companies of Russia": "تصنيف:شركات دفاعية في روسيا",
    "Category:Defunct companies of Russia": "تصنيف:شركات سابقة في روسيا",
    "Category:Defunct electric power companies of Russia": "تصنيف:شركات طاقة كهربائية سابقة في روسيا",
    "Category:Defunct energy companies of Russia": "تصنيف:شركات طاقة سابقة في روسيا",
    "Category:Defunct manufacturing companies of Russia": "تصنيف:شركات تصنيع سابقة في روسيا",
    "Category:Defunct oil companies of Russia": "تصنيف:شركات زيت الوقود سابقة في روسيا",
    "Category:Defunct shipping companies of Russia": "تصنيف:شركات نقل بحري سابقة في روسيا",
    "Category:Defunct transport companies of Russia": "تصنيف:شركات نقل سابقة في روسيا",
    "Category:Defunct video game companies of Russia": "تصنيف:شركات ألعاب فيديو سابقة في روسيا",
    "Category:Design companies of Russia": "تصنيف:شركات تصميم في روسيا",
    "Category:Drink companies of Russia": "تصنيف:شركات مشروبات في روسيا",
    "Category:Electrical engineering companies of Russia": "تصنيف:شركات هندسة كهربائية في روسيا",
    "Category:Electric power companies of Russia": "تصنيف:شركات طاقة كهربائية في روسيا",
    "Category:Electronics companies of Russia": "تصنيف:شركات إلكترونيات في روسيا",
    "Category:Energy companies of Russia": "تصنيف:شركات طاقة في روسيا",
    "Category:Engineering companies of Russia": "تصنيف:شركات هندسية في روسيا",
    "Category:Entertainment companies of Russia": "تصنيف:شركات ترفيه في روسيا",
    "Category:Financial services companies of Russia": "تصنيف:شركات خدمات مالية في روسيا",
    "Category:Food and drink companies of Russia": "تصنيف:شركات أطعمة ومشروبات في روسيا",
    "Category:Government-owned companies of Russia": "تصنيف:شركات مملوكة للحكومة في روسيا",
    "Category:Holding companies of Russia": "تصنيف:شركات قابضة في روسيا",
    "Category:Information technology companies of Russia": "تصنيف:شركات تقانة المعلومات في روسيا",
    "Category:Insurance companies of Russia": "تصنيف:شركات تأمين في روسيا",
    "Category:Investment companies of Russia": "تصنيف:شركات استثمارية في روسيا",
    "Category:Jewellery companies of Russia": "تصنيف:شركات مجوهرات في روسيا",
    "Category:Lists of companies of Russia": "تصنيف:قوائم شركات في روسيا",
    "Category:Manufacturing companies of Russia": "تصنيف:شركات تصنيع في روسيا",
    "Category:Mass media companies of Russia": "تصنيف:شركات وسائل إعلام في روسيا",
    "Category:Metal companies of Russia": "تصنيف:شركات معادن في روسيا",
    "Category:Mining companies of Russia": "تصنيف:شركات تعدين في روسيا",
    "Category:Mobile phone companies of Russia": "تصنيف:شركات هاتف محمول في روسيا",
    "Category:Natural gas companies of Russia": "تصنيف:شركات غاز طبيعي في روسيا",
    "Category:Nuclear companies of Russia": "تصنيف:شركات نووية في روسيا",
    "Category:Oil and gas companies of Russia": "تصنيف:شركات النفط والغاز في روسيا",
    "Category:Oil companies of Russia": "تصنيف:شركات زيت الوقود في روسيا",
    "Category:Pharmaceutical companies of Russia": "تصنيف:شركات أدوية في روسيا",
    "Category:Privately held companies of Russia": "تصنيف:شركات خاصة في روسيا",
    "Category:Publishing companies of Russia": "تصنيف:شركات نشر في روسيا",
    "Category:Pulp and paper companies of Russia": "تصنيف:شركات اللب والورق في روسيا",
    "Category:Railway companies of Russia": "تصنيف:شركات سكك حديدية في روسيا",
    "Category:Real estate companies of Russia": "تصنيف:شركات عقارية في روسيا",
    "Category:Retail companies of Russia": "تصنيف:شركات تجارة التجزئة في روسيا",
    "Category:Security companies of Russia": "تصنيف:شركات أمن في روسيا",
    "Category:Service companies of Russia": "تصنيف:شركات خدمات في روسيا",
    "Category:Shipbuilding companies of Russia": "تصنيف:شركات سفن في روسيا",
    "Category:Shipping companies of Russia": "تصنيف:شركات نقل بحري في روسيا",
    "Category:Software companies of Russia": "تصنيف:شركات برمجيات في روسيا",
    "Category:Technology companies of Russia": "تصنيف:شركات تقانة في روسيا",
    "Category:Television companies of Russia": "تصنيف:شركات تلفاز في روسيا",
    "Category:Transport companies of Russia": "تصنيف:شركات نقل في روسيا",
    "Category:Travel and holiday companies of Russia": "تصنيف:شركات السفر والعطلات في روسيا",
    "Category:Vehicle manufacturing companies of Russia": "تصنيف:شركات تصنيع مركبات في روسيا",
    "Category:Video game companies of Russia": "تصنيف:شركات ألعاب فيديو في روسيا",
    "Category:Water companies of Russia": "تصنيف:شركات مياه في روسيا",
}

to_test = [
    ("test_all_russia", data1),
    ("test_companies_russia", companies_data),
]


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.slow
def test_all_russia(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_peoples(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)
    dump_diff(diff_result, name)
    # dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



import functools

from ...helps import logger
from .countries_names_double_v2 import resolve_countries_names_double
from .nationalities_double_v2 import resolve_by_nats_double_v2


@functools.lru_cache(maxsize=None)
def new_relations_resolvers(category: str) -> str:
    logger.debug("--" * 20)
    logger.debug(f"<><><><><><> <<green>> Trying new_relations_resolvers for: {category=}")

    resolved_label = resolve_by_nats_double_v2(category) or resolve_countries_names_double(category)

    logger.info_if_or_debug(f"<<yellow>> end new_relations_resolvers: {category=}, {resolved_label=}", resolved_label)
    return resolved_label



"""Helpers for normalizing Arabic category titles.

The module exposes the :func:`fixlabel` entry point that performs a sequence of
regular-expression driven transformations. The transformations are heavily
localized for Arabic Wikipedia labels and rely on the constants defined in
:mod:`ArWikiCats.fix.fixlists`.
"""

from __future__ import annotations

import re
from typing import Iterable, Mapping

from ..helps import logger
from ..legacy_bots.make_bots.reg_lines import YEARS_REGEX_AR
from .fixlists import (
    ENDING_REPLACEMENTS,
    REPLACEMENTS,
    STARTING_REPLACEMENTS,
    YEAR_CATEGORY_LABELS,
)
from .mv_years import move_years
from .specific_normalizations import apply_category_specific_normalizations


def _apply_regex_replacements(text: str, replacements: Mapping[str, str]) -> str:
    """Sequentially apply regex-based replacements to ``text``.

    Args:
        text: The mutable text that should be normalized.
        replacements: Pairs of patterns and replacement strings to apply in
            insertion order.

    Returns:
        The normalized text.
    """

    for pattern, replacement in replacements.items():
        text = re.sub(pattern, replacement, text)
    return text


def _apply_prefix_replacements(text: str, replacements: Mapping[str, str]) -> str:
    """Replace matching prefixes using simple string slicing.

    Args:
        text: The text to normalise.
        replacements: Mapping of prefix strings to their normalized versions.

    Returns:
        The updated text.
    """

    for prefix, replacement in replacements.items():
        if text.startswith(prefix):
            text = replacement + text[len(prefix) :]
    return text


def _apply_suffix_replacements(text: str, replacements: Mapping[str, str]) -> str:
    """Replace matching suffixes using simple string slicing.

    Args:
        text: The text to normalise.
        replacements: Mapping of suffix strings to replacement values.

    Returns:
        The updated text with trailing whitespace removed.
    """

    for suffix, replacement in replacements.items():
        if text.endswith(suffix):
            text = text[: -len(suffix)] + replacement
            text = text.strip()
    return text


def _insert_year_preposition(text: str, categories: Iterable[str]) -> str:
    """Insert the preposition ``في`` when a year immediately follows a label.

    Args:
        text: The text to normalise.
        categories: Category names that should receive the additional
            preposition.

    Returns:
        The updated text with consistent year formatting.
    """

    for category in categories:
        pattern = rf"(\s*{re.escape(category)}) (\d+\s*|عقد \d+\s*|القرن \d+\s*)"
        text = re.sub(pattern, r"\g<1> في \g<2>", text)
    return text


def _apply_basic_normalizations(ar_label: str) -> str:
    """Apply the shared replacements that most labels require."""

    normalized = _apply_regex_replacements(ar_label, REPLACEMENTS)
    normalized = _insert_year_preposition(normalized, YEAR_CATEGORY_LABELS)
    normalized = _apply_prefix_replacements(normalized, STARTING_REPLACEMENTS)
    normalized = normalized.strip()
    normalized = _apply_suffix_replacements(normalized, ENDING_REPLACEMENTS)
    normalized = normalized.replace("نصب تذكارية لال", "نصب تذكارية لل")
    normalized = _apply_suffix_replacements(normalized, ENDING_REPLACEMENTS)
    return normalized


def _normalize_conflict_phrases(text: str) -> str:
    """Normalize phrases describing conflicts or geographic relations."""

    if match := re.match(r"^(الغزو \w+|\w+ الغزو \w+) في (\w+.*?)$", text):
        first_part = match.group(1)
        second_part = match.group(2)
        if second_part.startswith("ال"):
            second_part = second_part[1:]
        text = f"{first_part} ل{second_part}"
    return text


def _normalize_sub_regions(text: str) -> str:
    """Normalize sub-region terminology for a number of countries."""

    if any(country in text for country in ("اليابان", "يابانيون", "يابانيات")):
        text = re.sub(r"حسب الولاية", "حسب المحافظة", text)
    if "سريلانكي" in text or "سريلانكا" in text:
        text = re.sub(r"الإقليم", "المقاطعة", text)
        text = re.sub(r"أقاليم", "مقاطعات", text)
    if "تركيا" in text:
        text = re.sub(r"مديريات", "أقضية", text)
    if "جزائر" in text:
        text = re.sub(r"المقاطعة", "الإقليم", text)
        text = re.sub(r"مقاطعات", "أقاليم", text)
        text = re.sub(r"مديريات", "دوائر", text)
        text = re.sub(r"المديرية", "الدائرة", text)
    return text


def fix_it(ar_label: str, en_label: str) -> str:
    """Normalize an Arabic label based on heuristics and English context.

    Args:
        ar_label: The Arabic label that requires normalization.
        en_label: The English text associated with the label. The string is
            inspected for hints that determine specific Arabic replacements.

    Returns:
        A normalised Arabic label.
    """

    normalized_en = en_label.replace("_", " ").lower()

    ar_label = re.sub(r"\s+", " ", ar_label)
    ar_label = re.sub(r"\{\}", "", ar_label)

    # santa fe province
    if ar_label.endswith(" في") and " fe " not in f" {normalized_en.strip()} ":
        ar_label = ar_label[: -len(" في")]
        logger.debug(f"Removed stray 'في' from end of label: {ar_label}")

    if match := re.match(r".*(\d\d\d\d)\-(\d\d).*", ar_label, flags=re.IGNORECASE):
        start_year = match.group(1)
        end_year = match.group(2)
        ar_label = re.sub(f"{start_year}-{end_year}", f"{start_year}–{end_year}", ar_label)
        logger.debug(f"Replaced hyphen with en dash in range {start_year}-{end_year}")

    if re.sub(r"^\–\d+", "", ar_label) != ar_label:
        return ""

    ar_label = ar_label.strip()
    ar_label = _apply_basic_normalizations(ar_label)
    ar_label = re.sub(r"كأس العالم لكرة القدم (\d)", r"كأس العالم \g<1>", ar_label)
    ar_label = re.sub(r",", "،", ar_label)
    ar_label = re.sub(r"^(.*) تصفيات مؤهلة إلى (.*)$", r"تصفيات \g<1> مؤهلة إلى \g<2>", ar_label)
    ar_label = re.sub(r"تأسيسات (\d+.*)$", r"تأسيسات سنة \g<1>", ar_label)
    ar_label = re.sub(r"انحلالات (\d+.*)$", r"انحلالات سنة \g<1>", ar_label)
    ar_label = re.sub(r" من حسب ", " حسب ", ar_label)
    # ar_label = re.sub(r"لاعبو في " , "لاعبو ", ar_label)
    # ar_label = re.sub(r"لاعبو من " , "لاعبو " , ar_label)
    ar_label = apply_category_specific_normalizations(ar_label, normalized_en)
    ar_label = ar_label.strip()

    # if ar_label.endswith("أنتهت في"):
    # ar_label = re.sub(r"أنتهت في$" , "أنتهت" , ar_label ).strip()

    ar_label = _apply_basic_normalizations(ar_label)

    if "مبنية على" not in ar_label:
        ar_label = re.sub(r" على أفلام$", " في الأفلام", ar_label)

    ar_label = _normalize_sub_regions(ar_label)
    ar_label = ar_label.replace("(توضيح)", "")
    ar_label = ar_label.replace(" تأسست في ", " أسست في ")
    ar_label = ar_label.replace("ب201", "بسنة 201")
    ar_label = ar_label.replace("ب202", "بسنة 202")
    ar_label = ar_label.replace("على المريخ", "في المريخ")

    ar_label = " ".join(ar_label.strip().split())

    ar_label = re.sub(r"^شغب (\d+)", r"شغب في \g<1>", ar_label)
    ar_label = re.sub(r"قوائممتعلقة", "قوائم متعلقة", ar_label)
    ar_label = re.sub(r" في أصل ", " من أصل ", ar_label)
    ar_label = re.sub(r"حلقات قوائم", "قوائم حلقات", ar_label)
    ar_label = _normalize_conflict_phrases(ar_label)

    # if "لاعبو " and "سيدات" in label_old: fix it to لاعبات (use all())

    # if all(x in ar_label for x in ["لاعبو ", "سيدات"]): ar_label = ar_label.replace("لاعبو ", "لاعبات ")

    ar_label = ar_label.strip()
    return ar_label


def add_fee(text: str) -> str:
    """Ensure the preposition ``في`` precedes years in certain categories.

    Args:
        text: The label text that should be inspected.

    Returns:
        A label with normalized year phrasing.
    """

    categories = [
        "الآلة",
        "البلد",
        "البوابة",
        "الجنسية والمجموعة العرقية",
        "المجموعة العرقية",
        "الجنسية والمهنة",
        "الدين والجنسية",
        "المهنة والجنسية",
        "البلد أو اللغة",
        "النوع الفني",
        "الجنسية",
        "الحرب",
        "الدين",
        "السنة",
        "العقد",
        "القارة",
        "اللغة",
        "المدينة",
        "المنظمة",
        "المهنة",
        "الموقع",
        "النزاع",
        "الولاية",
    ]
    categories_expression = "|".join(categories)
    text = re.sub(rf" حسب\s({categories_expression}) ({YEARS_REGEX_AR})$", r" حسب \1 في \2", text)
    return text


def fixlabel(label_old: str, en: str = "") -> str:
    """Return a normalized Arabic label suitable for publication.

    Args:
        label_old: The original, possibly denormalized, Arabic label.
        out: When ``True`` emit an informational log when a label changes.
        en: Optional English context string that influences certain fixes.

    Returns:
        The normalized label string. An empty string indicates that the label
        was rejected by one of the validation steps.
    """
    logger.debug(f"fixlabel: Starting with label_old: {label_old=}| {en=}")
    original_label = label_old
    letters_regex = "[abcdefghijklmnopqrstuvwxyz]"
    if re.sub(letters_regex, "", label_old, flags=re.IGNORECASE) != label_old:
        return ""

    skip_labels = [
        "مراكز سياسية",
        "مشاعر معادية للإسرائيليون",
    ]

    if any(label in label_old for label in skip_labels):
        return ""

    if "لاعبات" in label_old and "مغتربون" in label_old:
        return ""

    label_old = label_old.strip()
    label_old = label_old.replace("_", " ")
    label_old = re.sub(r"تصنيف\:\s*", "", label_old)
    label_old = re.sub(r"تصنيف:", "", label_old)

    ar_label = label_old
    ar_label = fix_it(ar_label, en)
    ar_label = add_fee(ar_label)
    ar_label = move_years(ar_label)

    if "لاعبو" in ar_label and ("women's" in en or "womens" in en or "سيدات" in ar_label or "نسائية" in ar_label):
        ar_label = ar_label.replace("لاعبو", "لاعبات")

    if original_label != ar_label:
        logger.info(f'fixtitle: original_label:"{original_label}", after:"{ar_label}"')

    return ar_label


__all__ = [
    "add_fee",
    "fix_it",
    "fixlabel",
]



#!/usr/bin/python3
""" """

from .utils.json_dir import open_json_file

Taxons_table = {}
# ---
Taxons = open_json_file("taxonomy/Taxons.json") or {}
Taxons2 = open_json_file("taxonomy/Taxons2.json") or {}
# ---
Taxons.update(Taxons2)
# ---
Taxons_table.update(Taxons)
# ---
for tax, taxlab in Taxons.items():
    Taxons_table[f"{tax} of"] = taxlab
    Taxons_table[f"fossil {tax}"] = f"{taxlab} أحفورية"
    Taxons_table[f"fossil {tax} of"] = f"{taxlab} أحفورية"
# ---
for taxe, lab in Taxons2.items():
    Taxons_table[f"{taxe} of"] = f"{lab} في"

__all__ = [
    "Taxons_table",
]



#!/usr/bin/python3
"""
python3 core8/pwb.py -m cProfile -s ncalls make/make_bots.matables_bots/bot.py

"""

from ...helps import len_print
from ...translations import (
    ALBUMS_TYPE,
    Jobs_new,
)

typeTable_7: dict[str, str] = {
    "air force": "قوات جوية",
    "airlines accidents": "حوادث طيران",
    "aviation accident": "حوادث طيران",
    "aviation accidents": "حوادث طيران",
    "design institutions": "مؤسسات تصميم",
    "distance education institutions": "مؤسسات تعليم عن بعد",
    "executed-burning": "أعدموا شنقاً",
    "executed-decapitation": "أعدموا بقطع الرأس",
    "executed-firearm": "أعدموا بسلاح ناري",
    "executed-hanging": "أعدموا حرقاً",
    "executions": "إعدامات",
    "people executed by": "أشخاص أعدموا من قبل",
    "people executed-by-burning": "أشخاص أعدموا شنقاً",
    "people executed-by-decapitation": "أشخاص أعدموا بقطع الرأس",
    "people executed-by-firearm": "أشخاص أعدموا بسلاح ناري",
    "people executed-by-hanging": "أشخاص أعدموا حرقاً",
    "railway accident": "حوادث سكك حديد",
    "railway accidents": "حوادث سكك حديد",
    "road accidents": "حوادث طرق",
    "transport accident": "حوادث نقل",
    "transport accidents": "حوادث نقل",
    "transport disasters": "كوارث نقل",
}


def _create_pp_prefix(albums_typies: dict[str, str]) -> dict[str, str]:
    Pp_Priffix = {
        " memorials": "نصب {} التذكارية",
        " video albums": "ألبومات فيديو {}",
        " albums": "ألبومات {}",
        " cabinet": "مجلس وزراء {}",
        " administration cabinet members": "أعضاء مجلس وزراء إدارة {}",
        " administration personnel": "موظفو إدارة {}",
        " executive office": "مكتب {} التنفيذي",
    }

    for io in albums_typies:
        Pp_Priffix[f"{io} albums"] = "ألبومات %s {}" % albums_typies[io]

    return Pp_Priffix


def _make_players_keys() -> dict:
    players_keys = {}
    players_keys["women"] = "المرأة"

    players_keys.update({x.lower(): v for x, v in Jobs_new.items() if v})

    players_keys.update({x.lower(): v for x, v in typeTable_7.items()})

    players_keys["national sports teams"] = "منتخبات رياضية وطنية"
    players_keys["people"] = "أشخاص"

    return players_keys


players_new_keys = _make_players_keys()

Pp_Priffix = _create_pp_prefix(ALBUMS_TYPE)

cash_2022 = {
    "category:japan golf tour golfers": "تصنيف:لاعبو بطولة اليابان للغولف",
    "category:asian tour golfers": "تصنيف:لاعبو بطولة آسيا للغولف",
    "category:european tour golfers": "تصنيف:لاعبو بطولة أوروبا للغولف",
    "category:ladies european tour golfers": "تصنيف:لاعبات بطولة أوروبا للغولف للسيدات",
}
# ---
All_P17 = {}
Films_O_TT = {}


def add_to_new_players(en: str, ar: str) -> None:
    """Add a new English/Arabic player label pair to the cache."""
    if not en or not ar:
        return

    if not isinstance(en, str) or not isinstance(ar, str):
        return

    players_new_keys[en] = ar


def add_to_Films_O_TT(en: str, ar: str) -> None:
    """Add a new English/Arabic player label pair to the cache."""
    if not en or not ar:
        return

    if not isinstance(en, str) or not isinstance(ar, str):
        return

    Films_O_TT[en] = ar


len_print.data_len(
    "make_bots.matables_bots/bot.py",
    {
        "players_new_keys": players_new_keys,  # 99517
        "All_P17": All_P17,
    },
)

__all__ = [
    "cash_2022",
    "Films_O_TT",
    "players_new_keys",
    "add_to_new_players",
    "add_to_Films_O_TT",
    "All_P17",
    "Pp_Priffix",
]
